#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import contextlib
from unittest import mock

from heat.common import exception as exc
from heat.engine import stack
from heat.engine import template
from heat.tests import common
from heat.tests import utils


class SoftwareConfigTest(common.HeatTestCase):

    def setUp(self):
        super(SoftwareConfigTest, self).setUp()
        self.ctx = utils.dummy_context()
        self.properties = {
            'group': 'Heat::Shell',
            'inputs': [],
            'outputs': [],
            'options': {},
            'config': '#!/bin/bash'
        }
        self.stack = stack.Stack(
            self.ctx, 'software_config_test_stack',
            template.Template({
                'HeatTemplateFormatVersion': '2012-12-12',
                'Resources': {
                    'config_mysql': {
                        'Type': 'OS::Heat::SoftwareConfig',
                        'Properties': self.properties
                    }}}))
        self.config = self.stack['config_mysql']
        self.rpc_client = mock.MagicMock()
        self.config._rpc_client = self.rpc_client

        @contextlib.contextmanager
        def exc_filter(*args):
            try:
                yield
            except exc.NotFound:
                pass

        self.rpc_client.ignore_error_by_name.side_effect = exc_filter

    def test_handle_create(self):
        config_id = 'c8a19429-7fde-47ea-a42f-40045488226c'
        value = {'id': config_id}
        self.rpc_client.create_software_config.return_value = value
        self.config.handle_create()
        self.assertEqual(config_id, self.config.resource_id)

    def test_handle_delete(self):
        self.resource_id = None
        self.assertIsNone(self.config.handle_delete())
        config_id = 'c8a19429-7fde-47ea-a42f-40045488226c'
        self.config.resource_id = config_id
        self.rpc_client.delete_software_config.return_value = None
        self.assertIsNone(self.config.handle_delete())
        self.rpc_client.delete_software_config.side_effect = exc.NotFound
        self.assertIsNone(self.config.handle_delete())

    def test_resolve_attribute(self):
        self.assertIsNone(self.config._resolve_attribute('others'))
        self.config.resource_id = None
        self.assertIsNone(self.config._resolve_attribute('config'))
        self.config.resource_id = 'c8a19429-7fde-47ea-a42f-40045488226c'
        value = {'config': '#!/bin/bash'}
        self.rpc_client.show_software_config.return_value = value
        self.assertEqual(
            '#!/bin/bash', self.config._resolve_attribute('config'))
        self.rpc_client.show_software_config.side_effect = exc.NotFound
        self.assertIsNone(self.config._resolve_attribute('config'))



#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import collections
from unittest import mock

from manilaclient import exceptions

from heat.common import exception as heat_exception
from heat.tests import common
from heat.tests import utils


class ManilaClientPluginTest(common.HeatTestCase):
    scenarios = [
        ('share_type',
            dict(manager_name="share_types",
                 method_name="get_share_type")),
        ('share_network',
            dict(manager_name="share_networks",
                 method_name="get_share_network")),
        ('share_snapshot',
            dict(manager_name="share_snapshots",
                 method_name="get_share_snapshot")),
        ('security_service',
            dict(manager_name="security_services",
                 method_name="get_security_service")),
    ]

    def setUp(self):
        super(ManilaClientPluginTest, self).setUp()
        # mock client and plugin
        self.manila_client = mock.MagicMock()
        con = utils.dummy_context()
        c = con.clients
        self.manila_plugin = c.client_plugin('manila')
        self.manila_plugin.client = lambda: self.manila_client
        # prepare list of items to test search
        Item = collections.namedtuple('Item', ['id', 'name'])
        self.item_list = [
            Item(name="unique_name", id="unique_id"),
            Item(name="unique_id", id="i_am_checking_that_id_prior"),
            Item(name="duplicated_name", id="duplicate_test_one"),
            Item(name="duplicated_name", id="duplicate_test_second")]

    def test_create(self):
        context = utils.dummy_context()
        plugin = context.clients.client_plugin('manila')
        client = plugin.client()
        self.assertIsNotNone(client.security_services)
        self.assertEqual('http://server.test:5000/v3',
                         client.client.endpoint_url)

    def test_manila_get_method(self):
        # set item list as client output
        manager = getattr(self.manila_client, self.manager_name)
        manager.list.return_value = self.item_list
        # test that get_<method_name> is searching correctly
        get_method = getattr(self.manila_plugin, self.method_name)
        self.assertEqual(get_method("unique_id").name, "unique_name")
        self.assertRaises(heat_exception.EntityNotFound,
                          get_method, "non_exist")
        self.assertRaises(exceptions.NoUniqueMatch, get_method,
                          "duplicated_name")



#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy
from unittest import mock

from neutronclient.v2_0 import client as neutronclient

from heat.common import exception as heat_ex
from heat.common import short_id
from heat.common import template_format
from heat.engine.clients.os import neutron
from heat.engine.clients.os import nova
from heat.engine import node_data
from heat.engine.resources.openstack.nova import floatingip
from heat.engine import rsrc_defn
from heat.engine import scheduler
from heat.tests import common
from heat.tests.openstack.nova import fakes as fakes_nova
from heat.tests import utils


floating_ip_template = '''
{
    "heat_template_version": "2013-05-23",
    "resources": {
        "MyFloatingIP": {
            "type": "OS::Nova::FloatingIP",
            "properties": {
                "pool": "public"
            }
        }
    }
}
'''

floating_ip_template_with_assoc = '''
{
    "heat_template_version": "2013-05-23",
    "resources": {
        "MyFloatingIPAssociation": {
            "type": "OS::Nova::FloatingIPAssociation",
            "properties": {
                "server_id": "67dc62f9-efde-4c8b-94af-013e00f5dc57",
                "floating_ip": "fc68ea2c-b60b-4b4f-bd82-94ec81110766"
            }
        }
    }
}
'''


class NovaFloatingIPTest(common.HeatTestCase):
    def setUp(self):
        super(NovaFloatingIPTest, self).setUp()
        self.novaclient = fakes_nova.FakeClient()
        self.patchobject(nova.NovaClientPlugin, 'client',
                         return_value=self.novaclient)
        self.mock_create_fip = self.patchobject(neutronclient.Client,
                                                'create_floatingip')
        self.mock_upd_fip = self.patchobject(neutronclient.Client,
                                             'update_floatingip')
        self.mock_del_fip = self.patchobject(neutronclient.Client,
                                             'delete_floatingip')
        self.patchobject(neutron.NeutronClientPlugin,
                         'find_resourceid_by_name_or_id',
                         return_value='eeee')

    def mock_interface(self, port, ip):
        class MockIface(object):
            def __init__(self, port_id, fixed_ip):
                self.port_id = port_id
                self.fixed_ips = [{'ip_address': fixed_ip}]

        return MockIface(port, ip)

    def mock_create_floatingip(self):
        self.mock_create_fip.return_value = {'floatingip': {
            "status": "ACTIVE",
            "id": "fc68ea2c-b60b-4b4f-bd82-94ec81110766",
            'floating_network_id': 'eeee',
            "floating_ip_address": "11.0.0.1"
        }}

    def prepare_floating_ip(self):
        self.mock_create_floatingip()
        template = template_format.parse(floating_ip_template)
        self.stack = utils.parse_stack(template)
        defns = self.stack.t.resource_definitions(self.stack)

        return floatingip.NovaFloatingIp('MyFloatingIP',
                                         defns['MyFloatingIP'],
                                         self.stack)

    def prepare_floating_ip_assoc(self):
        return_server = self.novaclient.servers.list()[1]
        self.patchobject(self.novaclient.servers, 'get',
                         return_value=return_server)
        iface = self.mock_interface('aaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa',
                                    '1.2.3.4')
        self.patchobject(return_server, 'interface_list', return_value=[iface])

        template = template_format.parse(floating_ip_template_with_assoc)
        self.stack = utils.parse_stack(template)
        resource_defns = self.stack.t.resource_definitions(self.stack)
        floating_ip_assoc = resource_defns['MyFloatingIPAssociation']

        return floatingip.NovaFloatingIpAssociation(
            'MyFloatingIPAssociation', floating_ip_assoc, self.stack)

    def test_floating_ip_create(self):
        rsrc = self.prepare_floating_ip()
        rsrc.validate()
        scheduler.TaskRunner(rsrc.create)()

        self.assertEqual((rsrc.CREATE, rsrc.COMPLETE), rsrc.state)
        self.mock_create_fip.assert_called_with(
            {'floatingip': {'floating_network_id': 'eeee'}})
        self.assertEqual('fc68ea2c-b60b-4b4f-bd82-94ec81110766',
                         rsrc.FnGetRefId())
        self.assertEqual('11.0.0.1', rsrc.FnGetAtt('ip'))
        self.assertEqual('eeee', rsrc.FnGetAtt('pool'))

    def test_floating_ip_delete(self):
        rsrc = self.prepare_floating_ip()
        rsrc.validate()
        scheduler.TaskRunner(rsrc.create)()
        self.assertEqual((rsrc.CREATE, rsrc.COMPLETE), rsrc.state)
        scheduler.TaskRunner(rsrc.delete)()
        self.assertEqual((rsrc.DELETE, rsrc.COMPLETE), rsrc.state)
        self.mock_del_fip.assert_called_with(
            'fc68ea2c-b60b-4b4f-bd82-94ec81110766')

    def test_delete_floating_ip_assoc_successful_if_create_failed(self):
        rsrc = self.prepare_floating_ip_assoc()
        self.mock_upd_fip.side_effect = [fakes_nova.fake_exception(400)]

        rsrc.validate()
        self.assertRaises(heat_ex.ResourceFailure,
                          scheduler.TaskRunner(rsrc.create))
        self.assertEqual((rsrc.CREATE, rsrc.FAILED), rsrc.state)
        scheduler.TaskRunner(rsrc.delete)()
        self.assertEqual((rsrc.DELETE, rsrc.COMPLETE), rsrc.state)

    def test_floating_ip_assoc_create(self):
        rsrc = self.prepare_floating_ip_assoc()
        fip_request = {'floatingip': {
            'fixed_ip_address': '1.2.3.4',
            'port_id': 'aaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa'}
        }
        rsrc.validate()
        scheduler.TaskRunner(rsrc.create)()

        self.assertEqual((rsrc.CREATE, rsrc.COMPLETE), rsrc.state)

        self.mock_upd_fip.assert_called_with(
            'fc68ea2c-b60b-4b4f-bd82-94ec81110766', fip_request)
        self.assertIsNotNone(rsrc.id)
        self.assertEqual(rsrc.id, rsrc.resource_id)

    def test_floating_ip_assoc_delete(self):
        rsrc = self.prepare_floating_ip_assoc()
        rsrc.validate()
        scheduler.TaskRunner(rsrc.create)()
        self.assertEqual((rsrc.CREATE, rsrc.COMPLETE), rsrc.state)
        scheduler.TaskRunner(rsrc.delete)()
        self.assertEqual((rsrc.DELETE, rsrc.COMPLETE), rsrc.state)
        fip_request = {'floatingip': {
            'fixed_ip_address': None, 'port_id': None}}
        self.mock_upd_fip.assert_called_with(
            'fc68ea2c-b60b-4b4f-bd82-94ec81110766', fip_request)

    def create_delete_assoc_with_exc(self, exc):
        rsrc = self.prepare_floating_ip_assoc()
        rsrc.validate()
        scheduler.TaskRunner(rsrc.create)()
        self.assertEqual((rsrc.CREATE, rsrc.COMPLETE), rsrc.state)
        self.mock_upd_fip.side_effect = [exc]
        scheduler.TaskRunner(rsrc.delete)()
        self.assertEqual((rsrc.DELETE, rsrc.COMPLETE), rsrc.state)

    def test_floating_ip_assoc_delete_neutron_not_found(self):
        self.create_delete_assoc_with_exc(neutronclient.exceptions.NotFound())

    def test_floating_ip_assoc_delete_nova_not_found(self):
        self.create_delete_assoc_with_exc(fakes_nova.fake_exception(404))

    def test_floating_ip_assoc_update_server_id(self):
        rsrc = self.prepare_floating_ip_assoc()
        rsrc.validate()
        scheduler.TaskRunner(rsrc.create)()
        self.assertEqual((rsrc.CREATE, rsrc.COMPLETE), rsrc.state)

        # for update
        return_server = self.novaclient.servers.list()[2]
        self.patchobject(self.novaclient.servers, 'get',
                         return_value=return_server)
        iface = self.mock_interface('bbbbb-bbbb-bbbb-bbbbbbbbb',
                                    '4.5.6.7')
        self.patchobject(return_server, 'interface_list', return_value=[iface])

        # update with the new server_id
        props = copy.deepcopy(rsrc.properties.data)
        update_server_id = '2146dfbf-ba77-4083-8e86-d052f671ece5'
        props['server_id'] = update_server_id
        update_snippet = rsrc_defn.ResourceDefinition(rsrc.name, rsrc.type(),
                                                      props)
        scheduler.TaskRunner(rsrc.update, update_snippet)()
        self.assertEqual((rsrc.UPDATE, rsrc.COMPLETE), rsrc.state)

        fip_request = {'floatingip': {
            'fixed_ip_address': '1.2.3.4',
            'port_id': 'aaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa'}
        }
        fip_request_update = {'floatingip': {
            'fixed_ip_address': '4.5.6.7',
            'port_id': 'bbbbb-bbbb-bbbb-bbbbbbbbb'}
        }
        calls = [mock.call('fc68ea2c-b60b-4b4f-bd82-94ec81110766',
                           fip_request),
                 mock.call('fc68ea2c-b60b-4b4f-bd82-94ec81110766',
                           fip_request_update)]
        self.mock_upd_fip.assert_has_calls(calls)
        self.assertEqual(2, self.mock_upd_fip.call_count)

    def test_floating_ip_assoc_update_fl_ip(self):
        rsrc = self.prepare_floating_ip_assoc()
        rsrc.validate()
        scheduler.TaskRunner(rsrc.create)()
        self.assertEqual((rsrc.CREATE, rsrc.COMPLETE), rsrc.state)

        # update with the new floatingip
        props = copy.deepcopy(rsrc.properties.data)
        props['floating_ip'] = 'fc68ea2c-cccc-4b4f-bd82-94ec81110766'
        update_snippet = rsrc_defn.ResourceDefinition(rsrc.name, rsrc.type(),
                                                      props)
        scheduler.TaskRunner(rsrc.update, update_snippet)()
        self.assertEqual((rsrc.UPDATE, rsrc.COMPLETE), rsrc.state)

        fip_request = {'floatingip': {
            'fixed_ip_address': '1.2.3.4',
            'port_id': 'aaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa'}
        }
        fip_request_none = {'floatingip': {
            'fixed_ip_address': None, 'port_id': None}}
        calls = [mock.call('fc68ea2c-b60b-4b4f-bd82-94ec81110766',
                           fip_request),
                 mock.call('fc68ea2c-b60b-4b4f-bd82-94ec81110766',
                           fip_request_none),
                 mock.call('fc68ea2c-cccc-4b4f-bd82-94ec81110766',
                           fip_request)]
        self.mock_upd_fip.assert_has_calls(calls)
        self.assertEqual(3, self.mock_upd_fip.call_count)

    def test_floating_ip_assoc_update_both(self):
        rsrc = self.prepare_floating_ip_assoc()
        rsrc.validate()
        scheduler.TaskRunner(rsrc.create)()
        self.assertEqual((rsrc.CREATE, rsrc.COMPLETE), rsrc.state)
        # update with the new floatingip
        return_server = self.novaclient.servers.list()[2]
        self.patchobject(self.novaclient.servers, 'get',
                         return_value=return_server)
        iface = self.mock_interface('bbbbb-bbbb-bbbb-bbbbbbbbb',
                                    '4.5.6.7')
        self.patchobject(return_server, 'interface_list', return_value=[iface])
        props = copy.deepcopy(rsrc.properties.data)
        update_server_id = '2146dfbf-ba77-4083-8e86-d052f671ece5'
        props['server_id'] = update_server_id
        props['floating_ip'] = 'fc68ea2c-cccc-4b4f-bd82-94ec81110766'
        update_snippet = rsrc_defn.ResourceDefinition(rsrc.name, rsrc.type(),
                                                      props)
        scheduler.TaskRunner(rsrc.update, update_snippet)()
        self.assertEqual((rsrc.UPDATE, rsrc.COMPLETE), rsrc.state)
        fip_request = {'floatingip': {
            'port_id': 'aaaaaaaa-aaaa-aaaa-aaaa-aaaaaaaaaaaa',
            'fixed_ip_address': '1.2.3.4'}
        }
        fip_request_none = {'floatingip': {
            'port_id': None, 'fixed_ip_address': None}}
        fip_request_update = {'floatingip': {
            'port_id': 'bbbbb-bbbb-bbbb-bbbbbbbbb',
            'fixed_ip_address': '4.5.6.7'}
        }
        calls = [mock.call('fc68ea2c-b60b-4b4f-bd82-94ec81110766',
                           fip_request),
                 mock.call('fc68ea2c-b60b-4b4f-bd82-94ec81110766',
                           fip_request_none),
                 mock.call('fc68ea2c-cccc-4b4f-bd82-94ec81110766',
                           fip_request_update)]
        self.mock_upd_fip.assert_has_calls(calls)
        self.assertEqual(3, self.mock_upd_fip.call_count)

    def test_floating_ip_assoc_refid_rsrc_name(self):
        t = template_format.parse(floating_ip_template_with_assoc)
        stack = utils.parse_stack(t)
        rsrc = stack['MyFloatingIPAssociation']
        rsrc.id = '123'
        rsrc.uuid = '9bfb9456-3fe8-41f4-b318-9dba18eeef74'
        rsrc.action = 'CREATE'
        expected = '%s-%s-%s' % (rsrc.stack.name,
                                 rsrc.name,
                                 short_id.get_id(rsrc.uuid))
        self.assertEqual(expected, rsrc.FnGetRefId())

    def test_floating_ip_assoc_refid_rsrc_id(self):
        t = template_format.parse(floating_ip_template_with_assoc)
        stack = utils.parse_stack(t)
        rsrc = stack['MyFloatingIPAssociation']
        rsrc.resource_id = 'phy-rsrc-id'
        self.assertEqual('phy-rsrc-id', rsrc.FnGetRefId())

    def test_floating_ip_assoc_refid_convg_cache_data(self):
        t = template_format.parse(floating_ip_template_with_assoc)
        cache_data = {'MyFloatingIPAssociation': node_data.NodeData.from_dict({
            'uuid': mock.ANY,
            'id': mock.ANY,
            'action': 'CREATE',
            'status': 'COMPLETE',
            'reference_id': 'convg_xyz'
        })}
        stack = utils.parse_stack(t, cache_data=cache_data)
        rsrc = stack.defn['MyFloatingIPAssociation']
        self.assertEqual('convg_xyz', rsrc.FnGetRefId())



#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

from heat.engine.clients.os import nova
from heat.engine import stack
from heat.engine import template
from heat.tests import common
from heat.tests import utils

AGGREGATE_TEMPLATE = {
    'heat_template_version': '2013-05-23',
    'description': 'Heat Aggregate creation example',
    'resources': {
        'my_aggregate': {
            'type': 'OS::Nova::HostAggregate',
            'properties': {
                'name': 'host_aggregate',
                'availability_zone': 'nova',
                'hosts': ['host_1', 'host_2'],
                'metadata': {"availability_zone": "nova"}
            }
        }
    }
}


class NovaHostAggregateTest(common.HeatTestCase):
    def setUp(self):
        super(NovaHostAggregateTest, self).setUp()
        self.ctx = utils.dummy_context()

        self.stack = stack.Stack(
            self.ctx, 'nova_aggregate_test_stack',
            template.Template(AGGREGATE_TEMPLATE)
        )

        self.my_aggregate = self.stack['my_aggregate']
        nova_client = mock.MagicMock()
        self.novaclient = mock.MagicMock()
        self.my_aggregate.client = nova_client
        nova_client.return_value = self.novaclient
        self.aggregates = self.novaclient.aggregates

    def test_aggregate_handle_create(self):
        value = mock.MagicMock()
        aggregate_id = '927202df-1afb-497f-8368-9c2d2f26e5db'
        value.id = aggregate_id
        self.aggregates.create.return_value = value
        self.my_aggregate.handle_create()
        value.set_metadata.assert_called_once_with(
            {"availability_zone": "nova"})
        self.assertEqual(2, value.add_host.call_count)
        self.assertEqual(aggregate_id, self.my_aggregate.resource_id)

    def test_aggregate_handle_update_name(self):
        value = mock.MagicMock()
        self.aggregates.get.return_value = value
        prop_diff = {'name': 'new_host_aggregate',
                     'metadata': {'availability_zone': 'new_nova'},
                     'availability_zone': 'new_nova'}
        expected = {'name': 'new_host_aggregate',
                    'availability_zone': 'new_nova'}
        self.my_aggregate.handle_update(
            json_snippet=None, tmpl_diff=None, prop_diff=prop_diff
        )
        value.update.assert_called_once_with(expected)

    def test_aggregate_handle_update_hosts(self):
        ag = mock.MagicMock()
        ag.hosts = ['host_1', 'host_2']
        self.aggregates.get.return_value = ag
        prop_diff = {'hosts': ['host_1', 'host_3']}
        add_host_expected = 'host_3'
        remove_host_expected = 'host_2'
        self.my_aggregate.handle_update(
            json_snippet=None, tmpl_diff=None, prop_diff=prop_diff
        )
        self.assertEqual(0, ag.update.call_count)
        self.assertEqual(0, ag.set_metadata.call_count)
        ag.add_host.assert_called_once_with(add_host_expected)
        ag.remove_host.assert_called_once_with(remove_host_expected)

    def test_aggregate_handle_update_metadata(self):
        ag = mock.MagicMock()
        self.aggregates.get.return_value = ag
        prop_diff = {'metadata': {'availability_zone': 'nova3'}}
        set_metadata_expected = {'availability_zone': 'nova3'}
        self.my_aggregate.handle_update(
            json_snippet=None, tmpl_diff=None, prop_diff=prop_diff
        )
        self.assertEqual(0, ag.update.call_count)
        self.assertEqual(0, ag.add_host.call_count)
        self.assertEqual(0, ag.remove_host.call_count)
        ag.set_metadata.assert_called_once_with(set_metadata_expected)

    def test_aggregate_handle_delete_not_found(self):
        ag = mock.MagicMock()
        ag.id = '927202df-1afb-497f-8368-9c2d2f26e5db'
        ag.hosts = ['host_1']
        self.aggregates.get.side_effect = [nova.exceptions.NotFound(404)]
        self.my_aggregate.handle_delete()

    def test_aggregate_handle_delete(self):
        ag = mock.MagicMock()
        ag.id = '927202df-1afb-497f-8368-9c2d2f26e5db'
        ag.hosts = ['host_1']
        self.aggregates.get.return_value = ag
        self.aggregates.hosts = ag.hosts
        self.my_aggregate.resource_id = ag.id
        self.my_aggregate.handle_delete()
        self.assertEqual(1, self.aggregates.get.call_count)
        self.assertEqual(ag.hosts, self.aggregates.hosts)
        ag.remove_host.assert_called_once_with(ag.hosts[0])
        self.aggregates.delete.assert_called_once_with(ag.id)

    def test_aggregate_get_live_state(self):
        value = mock.MagicMock()
        value.to_dict.return_value = {
            'availability_zone': 'nova',
            'name': 'test',
            'hosts': ['ubuntu'],
            'metadata': {'test': 'test', 'availability_zone': 'nova'}
        }
        self.aggregates.get.return_value = value

        reality = self.my_aggregate.get_live_state(
            self.my_aggregate.properties)
        expected = {
            'availability_zone': 'nova',
            'name': 'test',
            'hosts': ['ubuntu'],
            'metadata': {'test': 'test', 'availability_zone': 'nova'}
        }

        self.assertEqual(set(expected.keys()), set(reality.keys()))
        for key in reality:
            self.assertEqual(expected[key], reality[key])



#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import ast
import tenacity

from oslo_log import log as logging

from heat.common import exception
from heat.objects import sync_point as sync_point_object

LOG = logging.getLogger(__name__)


KEY_SEPERATOR = ':'


def _dump_list(items, separator=', '):
    return separator.join(map(str, items))


def make_key(*components):
    assert len(components) >= 2
    return _dump_list(components, KEY_SEPERATOR)


def create(context, entity_id, traversal_id, is_update, stack_id):
    """Creates a sync point entry in DB."""
    values = {'entity_id': entity_id, 'traversal_id': traversal_id,
              'is_update': is_update, 'atomic_key': 0,
              'stack_id': stack_id, 'input_data': {}}
    return sync_point_object.SyncPoint.create(context, values)


def get(context, entity_id, traversal_id, is_update):
    """Retrieves a sync point entry from DB."""
    sync_point = sync_point_object.SyncPoint.get_by_key(context, entity_id,
                                                        traversal_id,
                                                        is_update)
    if sync_point is None:
        key = (entity_id, traversal_id, is_update)
        raise exception.EntityNotFound(entity='Sync Point', name=key)

    return sync_point


def delete_all(context, stack_id, traversal_id):
    """Deletes all sync points of a stack associated with a traversal_id."""
    return sync_point_object.SyncPoint.delete_all_by_stack_and_traversal(
        context, stack_id, traversal_id
    )


def update_input_data(context, entity_id, current_traversal,
                      is_update, atomic_key, input_data):
    rows_updated = sync_point_object.SyncPoint.update_input_data(
        context, entity_id, current_traversal, is_update, atomic_key,
        input_data)

    return rows_updated


def str_pack_tuple(t):
    return u'tuple:' + str(tuple(t))


def _str_unpack_tuple(s):
    s = s[s.index(':') + 1:]
    return ast.literal_eval(s)


def _deserialize(d):
    d2 = {}
    for k, v in d.items():
        if isinstance(k, str) and k.startswith(u'tuple:('):
            k = _str_unpack_tuple(k)
        if isinstance(v, dict):
            v = _deserialize(v)
        d2[k] = v
    return d2


def _serialize(d):
    d2 = {}
    for k, v in d.items():
        if isinstance(k, tuple):
            k = str_pack_tuple(k)
        if isinstance(v, dict):
            v = _serialize(v)
        d2[k] = v
    return d2


def deserialize_input_data(db_input_data):
    db_input_data = db_input_data.get('input_data')
    if not db_input_data:
        return {}

    return dict(_deserialize(db_input_data))


def serialize_input_data(input_data):
    return {'input_data': _serialize(input_data)}


def sync(cnxt, entity_id, current_traversal, is_update, propagate,
         predecessors, new_data):
    # Retry waits up to 60 seconds at most, with exponentially increasing
    # amounts of jitter per resource still outstanding
    wait_strategy = tenacity.wait_random_exponential(max=60)

    def init_jitter(existing_input_data):
        nconflicts = max(0, len(predecessors) - len(existing_input_data) - 1)
        # 10ms per potential conflict, up to a max of 10s in total
        return min(nconflicts, 1000) * 0.01

    @tenacity.retry(
        retry=tenacity.retry_if_result(lambda r: r is None),
        wait=wait_strategy
    )
    def _sync():
        sync_point = get(cnxt, entity_id, current_traversal, is_update)
        input_data = deserialize_input_data(sync_point.input_data)
        wait_strategy.multiplier = init_jitter(input_data)
        input_data.update(new_data)
        rows_updated = update_input_data(
            cnxt, entity_id, current_traversal, is_update,
            sync_point.atomic_key, serialize_input_data(input_data))
        return input_data if rows_updated else None

    input_data = _sync()
    waiting = predecessors - set(input_data)
    key = make_key(entity_id, current_traversal, is_update)
    if waiting:
        LOG.debug('[%s] Waiting %s: Got %s; still need %s',
                  key, entity_id, _dump_list(input_data), _dump_list(waiting))
    else:
        LOG.debug('[%s] Ready %s: Got %s',
                  key, entity_id, _dump_list(input_data))
        propagate(entity_id, serialize_input_data(input_data))



#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from heat.common import exception
from heat.common.i18n import _
from heat.engine import constraints
from heat.engine import properties
from heat.engine import resource
from heat.engine import support
from heat.engine import translation


class CinderQuota(resource.Resource):
    """A resource for creating cinder quotas.

    Cinder Quota is used to manage operational limits for projects. Currently,
    this resource can manage Cinder's gigabytes, snapshots, and volumes
    quotas.

    Note that default cinder security policy usage of this resource
    is limited to being used by administrators only. Administrators should be
    careful to create only one Cinder Quota resource per project, otherwise
    it will be hard for them to manage the quota properly.
    """

    support_status = support.SupportStatus(version='7.0.0')

    default_client_name = 'cinder'

    entity = 'quotas'

    required_service_extension = 'os-quota-sets'

    PROPERTIES = (PROJECT, GIGABYTES, VOLUMES, BACKUPS,
                  BACKUPS_GIGABYTES, SNAPSHOTS) = (
        'project', 'gigabytes', 'volumes',
        'backups', 'backup_gigabytes',
        'snapshots'
    )

    properties_schema = {
        PROJECT: properties.Schema(
            properties.Schema.STRING,
            _('OpenStack Keystone Project.'),
            required=True,
            constraints=[
                constraints.CustomConstraint('keystone.project')
            ]
        ),
        GIGABYTES: properties.Schema(
            properties.Schema.INTEGER,
            _('Quota for the amount of disk space (in Gigabytes). '
              'Setting the value to -1 removes the limit.'),
            constraints=[
                constraints.Range(min=-1),
            ],
            update_allowed=True
        ),
        VOLUMES: properties.Schema(
            properties.Schema.INTEGER,
            _('Quota for the number of volumes. '
              'Setting the value to -1 removes the limit.'),
            constraints=[
                constraints.Range(min=-1),
            ],
            update_allowed=True
        ),
        BACKUPS: properties.Schema(
            properties.Schema.INTEGER,
            _('Quota for the number of backups. '
              'Setting the value to -1 removes the limit.'),
            support_status=support.SupportStatus(version='16.0.0'),
            constraints=[
                constraints.Range(min=-1),
            ],
            update_allowed=True
        ),
        BACKUPS_GIGABYTES: properties.Schema(
            properties.Schema.INTEGER,
            _('Quota for the amount of backups disk space (in Gigabytes). '
              'Setting the value to -1 removes the limit.'),
            support_status=support.SupportStatus(version='16.0.0'),
            constraints=[
                constraints.Range(min=-1),
            ],
            update_allowed=True
        ),
        SNAPSHOTS: properties.Schema(
            properties.Schema.INTEGER,
            _('Quota for the number of snapshots. '
              'Setting the value to -1 removes the limit.'),
            constraints=[
                constraints.Range(min=-1),
            ],
            update_allowed=True
        )
    }

    def translation_rules(self, props):
        return [
            translation.TranslationRule(
                props,
                translation.TranslationRule.RESOLVE,
                [self.PROJECT],
                client_plugin=self.client_plugin('keystone'),
                finder='get_project_id')
        ]

    def handle_create(self):
        self._set_quota()
        self.resource_id_set(self.physical_resource_name())

    def handle_update(self, json_snippet, tmpl_diff, prop_diff):
        self._set_quota(json_snippet.properties(self.properties_schema,
                                                self.context))

    @classmethod
    def _validate_quota(cls, quota_property, quota_size, total_size):
        err_message = _("Invalid quota %(property)s value(s): %(value)s. "
                        "Can not be less than the current usage value(s): "
                        "%(total)s.")
        if quota_size < total_size:
            message_format = {'property': quota_property, 'value':
                              quota_size, 'total': total_size}
            raise ValueError(err_message % message_format)

    def validate_quotas(self, project, **kwargs):
        search_opts = {'all_tenants': True, 'project_id': project}
        volume_list = None
        backup_list = None
        snapshot_list = None
        for key, value in kwargs.copy().items():
            if value == -1:
                del kwargs[key]

        if self.GIGABYTES in kwargs:
            quota_size = kwargs[self.GIGABYTES]
            volume_list = self.client().volumes.list(search_opts=search_opts)
            snapshot_list = self.client().volume_snapshots.list(
                search_opts=search_opts)
            total_size = sum(item.size for item in (
                volume_list + snapshot_list))
            self._validate_quota(self.GIGABYTES, quota_size, total_size)

        if self.VOLUMES in kwargs:
            quota_size = kwargs[self.VOLUMES]
            if volume_list is None:
                volume_list = self.client().volumes.list(
                    search_opts=search_opts)
            total_size = len(volume_list)
            self._validate_quota(self.VOLUMES, quota_size, total_size)

        if self.BACKUPS in kwargs:
            quota_size = kwargs[self.BACKUPS]
            if backup_list is None:
                backup_list = self.client().backups.list(
                    search_opts=search_opts)
            total_size = len(backup_list)
            self._validate_quota(self.BACKUPS, quota_size, total_size)

        if self.BACKUPS_GIGABYTES in kwargs:
            quota_size = kwargs[self.BACKUPS_GIGABYTES]
            backup_list = self.client().backups.list(search_opts=search_opts)
            total_size = sum(item.size for item in (backup_list))
            self._validate_quota(self.BACKUPS_GIGABYTES,
                                 quota_size, total_size)

        if self.SNAPSHOTS in kwargs:
            quota_size = kwargs[self.SNAPSHOTS]
            if snapshot_list is None:
                snapshot_list = self.client().volume_snapshots.list(
                    search_opts=search_opts)
            total_size = len(snapshot_list)
            self._validate_quota(self.SNAPSHOTS, quota_size, total_size)

    def _set_quota(self, props=None):
        if props is None:
            props = self.properties

        kwargs = dict((k, v) for k, v in props.items()
                      if k != self.PROJECT and v is not None)
        # TODO(ricolin): Move this to stack validate stage. In some cases
        # we still can't get project or other properties form other resources
        # at validate stage.
        self.validate_quotas(props[self.PROJECT], **kwargs)
        self.client().quotas.update(props[self.PROJECT], **kwargs)

    def handle_delete(self):
        self.client().quotas.delete(self.properties[self.PROJECT])

    def validate(self):
        super(CinderQuota, self).validate()
        if sum(1 for p in self.properties.values() if p is not None) <= 1:
            raise exception.PropertyUnspecifiedError(self.GIGABYTES,
                                                     self.SNAPSHOTS,
                                                     self.VOLUMES,
                                                     self.BACKUPS,
                                                     self.BACKUPS_GIGABYTES)


def resource_mapping():
    return {
        'OS::Cinder::Quota': CinderQuota
    }



#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

from openstack import exceptions

from heat.engine.clients.os import openstacksdk
from heat.tests import common
from heat.tests import utils


class OpenStackSDKPluginTest(common.HeatTestCase):
    @mock.patch('openstack.connection.Connection')
    def setUp(self, mock_connection):
        super(OpenStackSDKPluginTest, self).setUp()
        context = utils.dummy_context()
        self.plugin = context.clients.client_plugin('openstack')

    def test_create(self):
        client = self.plugin.client()
        self.assertIsNotNone(client.network.segments)

    def test_is_not_found(self):
        self.assertFalse(self.plugin.is_not_found(
            exceptions.HttpException(http_status=400)))
        self.assertFalse(self.plugin.is_not_found(Exception))
        self.assertTrue(self.plugin.is_not_found(
            exceptions.NotFoundException(http_status=404)))


class SegmentConstraintTest(common.HeatTestCase):

    def setUp(self):
        super(SegmentConstraintTest, self).setUp()
        self.ctx = utils.dummy_context()
        self.mock_find_segment = mock.Mock()
        self.ctx.clients.client_plugin(
            'openstack').find_network_segment = self.mock_find_segment
        self.constraint = openstacksdk.SegmentConstraint()

    def test_validation(self):
        self.mock_find_segment.side_effect = [
            "seg1", exceptions.ResourceNotFound(),
            exceptions.DuplicateResource()]
        self.assertTrue(self.constraint.validate("foo", self.ctx))
        self.assertFalse(self.constraint.validate("bar", self.ctx))
        self.assertFalse(self.constraint.validate("baz", self.ctx))



# Copyright (c) 2016 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from unittest import mock
import uuid

import eventlet
from oslo_config import cfg

from heat.common import exception
from heat.engine import check_resource
from heat.engine import dependencies
from heat.engine import resource
from heat.engine import scheduler
from heat.engine import stack
from heat.engine import sync_point
from heat.engine import worker
from heat.rpc import api as rpc_api
from heat.rpc import worker_client
from heat.tests import common
from heat.tests.engine import tools
from heat.tests import utils


@mock.patch.object(check_resource, 'check_stack_complete')
@mock.patch.object(check_resource, 'propagate_check_resource')
@mock.patch.object(check_resource, 'check_resource_cleanup')
@mock.patch.object(check_resource, 'check_resource_update')
class CheckWorkflowUpdateTest(common.HeatTestCase):
    @mock.patch.object(worker_client.WorkerClient, 'check_resource',
                       lambda *_: None)
    def setUp(self):
        super(CheckWorkflowUpdateTest, self).setUp()
        thread_group_mgr = mock.Mock()
        cfg.CONF.set_default('convergence_engine', True)
        self.worker = worker.WorkerService('host-1',
                                           'topic-1',
                                           'engine_id',
                                           thread_group_mgr)
        self.cr = check_resource.CheckResource(self.worker.engine_id,
                                               self.worker._rpc_client,
                                               self.worker.thread_group_mgr,
                                               mock.Mock(), {})
        self.worker._rpc_client = worker_client.WorkerClient()
        self.ctx = utils.dummy_context()
        self.stack = tools.get_stack(
            'check_workflow_create_stack', self.ctx,
            template=tools.string_template_five, convergence=True)
        self.stack.converge_stack(self.stack.t)
        self.resource = self.stack['A']
        self.is_update = True
        self.graph_key = (self.resource.id, self.is_update)
        self.orig_load_method = stack.Stack.load
        stack.Stack.load = mock.Mock(return_value=self.stack)

    def tearDown(self):
        super(CheckWorkflowUpdateTest, self).tearDown()
        stack.Stack.load = self.orig_load_method

    def test_resource_not_available(
            self, mock_cru, mock_crc, mock_pcr, mock_csc):
        self.worker.check_resource(
            self.ctx, 'non-existant-id', self.stack.current_traversal, {},
            True, None)
        for mocked in [mock_cru, mock_crc, mock_pcr, mock_csc]:
            self.assertFalse(mocked.called)

    @mock.patch.object(worker.WorkerService, '_retrigger_replaced')
    def test_stale_traversal(
            self, mock_rnt, mock_cru, mock_crc, mock_pcr, mock_csc):
        self.worker.check_resource(self.ctx, self.resource.id,
                                   'stale-traversal', {}, True, None)
        self.assertTrue(mock_rnt.called)

    def test_is_update_traversal(
            self, mock_cru, mock_crc, mock_pcr, mock_csc):
        self.worker.check_resource(
            self.ctx, self.resource.id, self.stack.current_traversal, {},
            self.is_update, None)
        mock_cru.assert_called_once_with(self.resource,
                                         self.resource.stack.t.id,
                                         set(), self.worker.engine_id,
                                         mock.ANY, mock.ANY)
        self.assertFalse(mock_crc.called)

        expected_calls = []
        for req, fwd in self.stack.convergence_dependencies.leaves():
            expected_calls.append(
                (mock.call.worker.propagate_check_resource.
                    assert_called_once_with(
                        self.ctx, mock.ANY, mock.ANY,
                        self.stack.current_traversal, mock.ANY,
                        self.graph_key, {}, self.is_update)))
        mock_csc.assert_called_once_with(
            self.ctx, mock.ANY, self.stack.current_traversal,
            self.resource.id,
            mock.ANY, True)

    @mock.patch.object(resource.Resource, 'load')
    @mock.patch.object(resource.Resource, 'make_replacement')
    @mock.patch.object(stack.Stack, 'time_remaining')
    def test_is_update_traversal_raise_update_replace(
            self, tr, mock_mr, mock_load, mock_cru, mock_crc, mock_pcr,
            mock_csc):
        mock_load.return_value = self.resource, self.stack, self.stack
        mock_cru.side_effect = resource.UpdateReplace
        tr.return_value = 317
        self.worker.check_resource(
            self.ctx, self.resource.id, self.stack.current_traversal, {},
            self.is_update, None)
        mock_cru.assert_called_once_with(self.resource,
                                         self.resource.stack.t.id,
                                         set(), self.worker.engine_id,
                                         mock.ANY, mock.ANY)
        self.assertTrue(mock_mr.called)
        self.assertFalse(mock_crc.called)
        self.assertFalse(mock_pcr.called)
        self.assertFalse(mock_csc.called)

    @mock.patch.object(check_resource.CheckResource,
                       '_stale_resource_needs_retry')
    @mock.patch.object(stack.Stack, 'time_remaining')
    def test_is_update_traversal_raise_update_inprogress(
            self, tr, mock_tsl, mock_cru, mock_crc, mock_pcr,
            mock_csc):
        mock_cru.side_effect = exception.UpdateInProgress
        self.worker.engine_id = 'some-thing-else'
        mock_tsl.return_value = True
        tr.return_value = 317
        self.worker.check_resource(
            self.ctx, self.resource.id, self.stack.current_traversal, {},
            self.is_update, None)
        mock_cru.assert_called_once_with(self.resource,
                                         self.resource.stack.t.id,
                                         set(), self.worker.engine_id,
                                         mock.ANY, mock.ANY)
        self.assertFalse(mock_crc.called)
        self.assertFalse(mock_pcr.called)
        self.assertFalse(mock_csc.called)

    @mock.patch.object(resource.Resource, 'state_set')
    def test_stale_resource_retry(
            self, mock_ss, mock_cru, mock_crc, mock_pcr, mock_csc):
        current_template_id = self.resource.current_template_id
        res = self.cr._stale_resource_needs_retry(self.ctx,
                                                  self.resource,
                                                  current_template_id)
        self.assertTrue(res)
        mock_ss.assert_not_called()

    @mock.patch.object(resource.Resource, 'state_set')
    def test_try_steal_lock_alive(
            self, mock_ss, mock_cru, mock_crc, mock_pcr, mock_csc):
        res = self.cr._stale_resource_needs_retry(self.ctx,
                                                  self.resource,
                                                  str(uuid.uuid4()))
        self.assertFalse(res)
        mock_ss.assert_not_called()

    @mock.patch.object(check_resource.listener_client, 'EngineListenerClient')
    @mock.patch.object(check_resource.resource_objects.Resource, 'get_obj')
    @mock.patch.object(resource.Resource, 'state_set')
    def test_try_steal_lock_dead(
            self, mock_ss, mock_get, mock_elc, mock_cru, mock_crc, mock_pcr,
            mock_csc):
        fake_res = mock.Mock()
        fake_res.engine_id = 'some-thing-else'
        mock_get.return_value = fake_res
        mock_elc.return_value.is_alive.return_value = False
        current_template_id = self.resource.current_template_id
        res = self.cr._stale_resource_needs_retry(self.ctx,
                                                  self.resource,
                                                  current_template_id)
        self.assertTrue(res)
        mock_ss.assert_called_once_with(self.resource.action,
                                        resource.Resource.FAILED,
                                        mock.ANY)

    @mock.patch.object(check_resource.listener_client, 'EngineListenerClient')
    @mock.patch.object(check_resource.resource_objects.Resource, 'get_obj')
    @mock.patch.object(resource.Resource, 'state_set')
    def test_try_steal_lock_not_dead(
            self, mock_ss, mock_get, mock_elc, mock_cru, mock_crc, mock_pcr,
            mock_csc):
        fake_res = mock.Mock()
        fake_res.engine_id = self.worker.engine_id
        mock_get.return_value = fake_res
        mock_elc.return_value.is_alive.return_value = True
        current_template_id = self.resource.current_template_id
        res = self.cr._stale_resource_needs_retry(self.ctx,
                                                  self.resource,
                                                  current_template_id)
        self.assertFalse(res)
        mock_ss.assert_not_called()

    @mock.patch.object(stack.Stack, 'rollback')
    def test_resource_update_failure_sets_stack_state_as_failed(
            self, mock_tr, mock_cru, mock_crc, mock_pcr, mock_csc):
        self.stack.state_set(self.stack.UPDATE, self.stack.IN_PROGRESS, '')
        self.resource.state_set(self.resource.UPDATE,
                                self.resource.IN_PROGRESS)
        dummy_ex = exception.ResourceNotAvailable(
            resource_name=self.resource.name)
        mock_cru.side_effect = exception.ResourceFailure(
            dummy_ex, self.resource, action=self.resource.UPDATE)
        self.worker.check_resource(self.ctx, self.resource.id,
                                   self.stack.current_traversal, {},
                                   self.is_update, None)
        s = self.stack.load(self.ctx, stack_id=self.stack.id)
        self.assertEqual((s.UPDATE, s.FAILED), (s.action, s.status))
        self.assertEqual('Resource UPDATE failed: '
                         'ResourceNotAvailable: resources.A: The Resource (A)'
                         ' is not available.', s.status_reason)

    @mock.patch.object(stack.Stack, 'rollback')
    def test_resource_cleanup_failure_sets_stack_state_as_failed(
            self, mock_tr, mock_cru, mock_crc, mock_pcr, mock_csc):
        self.is_update = False  # invokes check_resource_cleanup
        self.stack.state_set(self.stack.UPDATE, self.stack.IN_PROGRESS, '')
        self.resource.state_set(self.resource.UPDATE,
                                self.resource.IN_PROGRESS)
        dummy_ex = exception.ResourceNotAvailable(
            resource_name=self.resource.name)
        mock_crc.side_effect = exception.ResourceFailure(
            dummy_ex, self.resource, action=self.resource.UPDATE)
        self.worker.check_resource(self.ctx, self.resource.id,
                                   self.stack.current_traversal, {},
                                   self.is_update, None)
        s = self.stack.load(self.ctx, stack_id=self.stack.id)
        self.assertEqual((s.UPDATE, s.FAILED), (s.action, s.status))
        self.assertEqual('Resource UPDATE failed: '
                         'ResourceNotAvailable: resources.A: The Resource (A)'
                         ' is not available.', s.status_reason)

    def test_resource_update_failure_triggers_rollback_if_enabled(
            self, mock_cru, mock_crc, mock_pcr, mock_csc):
        mock_tr = self.stack.rollback = mock.Mock(return_value=None)
        self.stack.disable_rollback = False
        self.stack.store()
        dummy_ex = exception.ResourceNotAvailable(
            resource_name=self.resource.name)
        mock_cru.side_effect = exception.ResourceFailure(
            dummy_ex, self.resource, action=self.resource.UPDATE)
        self.worker.check_resource(self.ctx, self.resource.id,
                                   self.stack.current_traversal, {},
                                   self.is_update, None)
        self.assertTrue(mock_tr.called)
        mock_tr.assert_called_once_with()

    def test_resource_cleanup_failure_triggers_rollback_if_enabled(
            self, mock_cru, mock_crc, mock_pcr, mock_csc):
        mock_tr = self.stack.rollback = mock.Mock(return_value=None)
        self.is_update = False  # invokes check_resource_cleanup
        self.stack.disable_rollback = False
        self.stack.store()
        dummy_ex = exception.ResourceNotAvailable(
            resource_name=self.resource.name)
        mock_crc.side_effect = exception.ResourceFailure(
            dummy_ex, self.resource, action=self.resource.UPDATE)
        self.worker.check_resource(self.ctx, self.resource.id,
                                   self.stack.current_traversal, {},
                                   self.is_update, None)
        mock_tr.assert_called_once_with()

    @mock.patch.object(stack.Stack, 'rollback')
    def test_rollback_is_not_triggered_on_rollback_disabled_stack(
            self, mock_tr, mock_cru, mock_crc, mock_pcr, mock_csc):
        self.stack.disable_rollback = True
        self.stack.store()
        dummy_ex = exception.ResourceNotAvailable(
            resource_name=self.resource.name)
        mock_cru.side_effect = exception.ResourceFailure(
            dummy_ex, self.resource, action=self.stack.CREATE)
        self.worker.check_resource(self.ctx, self.resource.id,
                                   self.stack.current_traversal, {},
                                   self.is_update, None)
        self.assertFalse(mock_tr.called)

    @mock.patch.object(stack.Stack, 'rollback')
    def test_rollback_not_re_triggered_for_a_rolling_back_stack(
            self, mock_tr, mock_cru, mock_crc, mock_pcr, mock_csc):
        self.stack.disable_rollback = False
        self.stack.action = self.stack.ROLLBACK
        self.stack.status = self.stack.IN_PROGRESS
        self.stack.store()
        dummy_ex = exception.ResourceNotAvailable(
            resource_name=self.resource.name)
        mock_cru.side_effect = exception.ResourceFailure(
            dummy_ex, self.resource, action=self.stack.CREATE)
        self.worker.check_resource(self.ctx, self.resource.id,
                                   self.stack.current_traversal, {},
                                   self.is_update, None)
        self.assertFalse(mock_tr.called)

    def test_resource_update_failure_purges_db_for_stack_failure(
            self, mock_cru, mock_crc, mock_pcr, mock_csc):
        self.stack.disable_rollback = True
        self.stack.store()
        self.stack.purge_db = mock.Mock()
        dummy_ex = exception.ResourceNotAvailable(
            resource_name=self.resource.name)
        mock_cru.side_effect = exception.ResourceFailure(
            dummy_ex, self.resource, action=self.resource.UPDATE)
        self.worker.check_resource(self.ctx, self.resource.id,
                                   self.stack.current_traversal, {},
                                   self.is_update, None)
        self.assertTrue(self.stack.purge_db.called)

    def test_resource_cleanup_failure_purges_db_for_stack_failure(
            self, mock_cru, mock_crc, mock_pcr, mock_csc):
        self.is_update = False
        self.stack.disable_rollback = True
        self.stack.store()
        self.stack.purge_db = mock.Mock()
        dummy_ex = exception.ResourceNotAvailable(
            resource_name=self.resource.name)
        mock_crc.side_effect = exception.ResourceFailure(
            dummy_ex, self.resource, action=self.resource.UPDATE)
        self.worker.check_resource(self.ctx, self.resource.id,
                                   self.stack.current_traversal, {},
                                   self.is_update, None)
        self.assertTrue(self.stack.purge_db.called)

    @mock.patch.object(check_resource.CheckResource,
                       'retrigger_check_resource')
    @mock.patch.object(stack.Stack, 'load')
    def test_initiate_propagate_rsrc_retriggers_check_rsrc_on_new_stack_update(
            self, mock_stack_load, mock_rcr, mock_cru, mock_crc, mock_pcr,
            mock_csc):
        key = sync_point.make_key(self.resource.id,
                                  self.stack.current_traversal,
                                  self.is_update)
        mock_pcr.side_effect = exception.EntityNotFound(entity='Sync Point',
                                                        name=key)
        updated_stack = stack.Stack(self.ctx, self.stack.name, self.stack.t,
                                    self.stack.id,
                                    current_traversal='some_newy_trvl_uuid')
        mock_stack_load.return_value = updated_stack
        self.cr._initiate_propagate_resource(self.ctx, self.resource.id,
                                             self.stack.current_traversal,
                                             self.is_update, self.resource,
                                             self.stack)
        mock_rcr.assert_called_once_with(self.ctx,
                                         self.resource.id, updated_stack)

    def test_check_stack_complete_is_invoked_for_replaced_resource(
            self, mock_cru, mock_crc, mock_pcr, mock_csc):
        resC = self.stack['C']
        # lets say C is update-replaced
        is_update = True
        trav_id = self.stack.current_traversal
        replacementC_id = resC.make_replacement(self.stack.t.id,
                                                set(resC.requires))
        replacementC, stack, _ = resource.Resource.load(self.ctx,
                                                        replacementC_id,
                                                        trav_id,
                                                        is_update, {})
        self.cr._initiate_propagate_resource(self.ctx, replacementC_id,
                                             trav_id,
                                             is_update, replacementC,
                                             self.stack)
        # check_stack_complete should be called with resC.id not
        # replacementC.id
        mock_csc.assert_called_once_with(self.ctx, self.stack,
                                         trav_id,
                                         resC.id, mock.ANY,
                                         is_update)

    @mock.patch.object(sync_point, 'sync')
    def test_retrigger_check_resource(self, mock_sync, mock_cru, mock_crc,
                                      mock_pcr, mock_csc):
        resC = self.stack['C']
        # A, B are predecessors to C when is_update is True
        expected_predecessors = {(self.stack['A'].id, True),
                                 (self.stack['B'].id, True)}
        self.cr.retrigger_check_resource(self.ctx, resC.id, self.stack)
        mock_pcr.assert_called_once_with(self.ctx, mock.ANY, resC.id,
                                         self.stack.current_traversal,
                                         mock.ANY, (resC.id, True), None,
                                         True, None)
        call_args, call_kwargs = mock_pcr.call_args
        actual_predecessors = call_args[4]
        self.assertCountEqual(expected_predecessors, actual_predecessors)

    def test_update_retrigger_check_resource_new_traversal_deletes_rsrc(
            self, mock_cru, mock_crc, mock_pcr, mock_csc):
        # mock dependencies to indicate a rsrc with id 2 is not present
        # in latest traversal
        self.stack._convg_deps = dependencies.Dependencies([
            [(1, False), (1, True)], [(2, False), None]])
        # simulate rsrc 2 completing its update for old traversal
        # and calling rcr
        self.cr.retrigger_check_resource(self.ctx, 2, self.stack)
        # Ensure that pcr was called with proper delete traversal
        mock_pcr.assert_called_once_with(self.ctx, mock.ANY, 2,
                                         self.stack.current_traversal,
                                         mock.ANY, (2, False), None,
                                         False, None)

    def test_delete_retrigger_check_resource_new_traversal_updates_rsrc(
            self, mock_cru, mock_crc, mock_pcr, mock_csc):
        # mock dependencies to indicate a rsrc with id 2 has an update
        # in latest traversal
        self.stack._convg_deps = dependencies.Dependencies([
            [(1, False), (1, True)], [(2, False), (2, True)]])
        # simulate rsrc 2 completing its delete for old traversal
        # and calling rcr
        self.cr.retrigger_check_resource(self.ctx, 2, self.stack)
        # Ensure that pcr was called with proper delete traversal
        mock_pcr.assert_called_once_with(self.ctx, mock.ANY, 2,
                                         self.stack.current_traversal,
                                         mock.ANY, (2, True), None,
                                         True, None)

    @mock.patch.object(stack.Stack, 'purge_db')
    def test_handle_failure(self, mock_purgedb, mock_cru, mock_crc, mock_pcr,
                            mock_csc):
        self.stack.mark_failed('dummy-reason')
        mock_purgedb.assert_called_once_with()
        self.assertEqual('dummy-reason', self.stack.status_reason)

    def test_handle_failure_rollback(self, mock_cru, mock_crc,
                                     mock_pcr, mock_csc):
        mock_tr = self.stack.rollback = mock.Mock(return_value=None)
        self.stack.disable_rollback = False
        self.stack.state_set(self.stack.UPDATE, self.stack.IN_PROGRESS, '')
        self.stack.mark_failed('dummy-reason')
        mock_tr.assert_called_once_with()

    @mock.patch.object(stack.Stack, 'purge_db')
    @mock.patch.object(stack.Stack, 'state_set')
    @mock.patch.object(check_resource.CheckResource,
                       'retrigger_check_resource')
    @mock.patch.object(stack.Stack, 'rollback')
    def test_handle_rsrc_failure_when_update_fails(
            self, mock_tr, mock_rcr, mock_ss, mock_pdb, mock_cru, mock_crc,
            mock_pcr, mock_csc):
        # Emulate failure
        mock_ss.return_value = False
        self.cr._handle_resource_failure(self.ctx, self.is_update,
                                         self.resource.id, self.stack,
                                         'dummy-reason')
        self.assertTrue(mock_ss.called)
        self.assertFalse(mock_rcr.called)
        self.assertFalse(mock_pdb.called)
        self.assertFalse(mock_tr.called)

    @mock.patch.object(stack.Stack, 'purge_db')
    @mock.patch.object(stack.Stack, 'state_set')
    @mock.patch.object(check_resource.CheckResource,
                       'retrigger_check_resource')
    @mock.patch.object(stack.Stack, 'rollback')
    def test_handle_rsrc_failure_when_update_fails_different_traversal(
            self, mock_tr, mock_rcr, mock_ss, mock_pdb, mock_cru, mock_crc,
            mock_pcr, mock_csc):
        # Emulate failure
        mock_ss.return_value = False

        # Emulate new traversal
        new_stack = tools.get_stack('check_workflow_create_stack', self.ctx,
                                    template=tools.string_template_five,
                                    convergence=True)
        new_stack.current_traversal = 'new_traversal'
        stack.Stack.load = mock.Mock(return_value=new_stack)

        self.cr._handle_resource_failure(self.ctx, self.is_update,
                                         self.resource.id,
                                         self.stack, 'dummy-reason')
        # Ensure retrigger called
        self.assertTrue(mock_rcr.called)
        self.assertTrue(mock_ss.called)
        self.assertFalse(mock_pdb.called)
        self.assertFalse(mock_tr.called)

    def test_handle_stack_timeout(self, mock_cru, mock_crc, mock_pcr,
                                  mock_csc):
        mock_mf = self.stack.mark_failed = mock.Mock(return_value=True)
        self.cr._handle_stack_timeout(self.ctx, self.stack)
        mock_mf.assert_called_once_with(u'Timed out')

    def test_do_check_resource_marks_stack_as_failed_if_stack_timesout(
            self, mock_cru, mock_crc, mock_pcr, mock_csc):
        mock_mf = self.stack.mark_failed = mock.Mock(return_value=True)
        mock_cru.side_effect = scheduler.Timeout(None, 60)
        self.is_update = True
        self.cr._do_check_resource(self.ctx, self.stack.current_traversal,
                                   self.stack.t, {}, self.is_update,
                                   self.resource, self.stack, {})
        mock_mf.assert_called_once_with(u'Timed out')

    @mock.patch.object(check_resource.CheckResource,
                       '_handle_stack_timeout')
    def test_do_check_resource_ignores_timeout_for_new_update(
            self, mock_hst, mock_cru, mock_crc, mock_pcr, mock_csc):
        # Ensure current_traversal is check before marking the stack as
        # failed due to time-out.
        mock_cru.side_effect = scheduler.Timeout(None, 60)
        self.is_update = True
        old_traversal = self.stack.current_traversal
        self.stack.current_traversal = 'new_traversal'
        self.cr._do_check_resource(self.ctx, old_traversal,
                                   self.stack.t, {}, self.is_update,
                                   self.resource, self.stack, {})
        self.assertFalse(mock_hst.called)

    @mock.patch.object(stack.Stack, 'has_timed_out')
    @mock.patch.object(check_resource.CheckResource,
                       '_handle_stack_timeout')
    def test_check_resource_handles_timeout(self, mock_hst, mock_to, mock_cru,
                                            mock_crc, mock_pcr, mock_csc):
        mock_to.return_value = True
        self.worker.check_resource(self.ctx, self.resource.id,
                                   self.stack.current_traversal,
                                   {}, self.is_update, {})
        self.assertTrue(mock_hst.called)

    def test_check_resource_does_not_propagate_on_cancel(
            self, mock_cru, mock_crc, mock_pcr, mock_csc):
        # ensure when check_resource is cancelled, the next set of
        # resources are not propagated.
        mock_cru.side_effect = check_resource.CancelOperation
        self.worker.check_resource(self.ctx, self.resource.id,
                                   self.stack.current_traversal,
                                   {}, self.is_update, {})
        self.assertFalse(mock_pcr.called)
        self.assertFalse(mock_csc.called)

    @mock.patch.object(resource.Resource, 'load')
    def test_requires(self, mock_load, mock_cru, mock_crc, mock_pcr, mock_csc):
        mock_load.return_value = self.resource, self.stack, self.stack
        res_data = {(1, True): {u'id': 5, u'name': 'A', 'attrs': {}},
                    (2, True): {u'id': 3, u'name': 'B', 'attrs': {}}}
        self.worker.check_resource(self.ctx, self.resource.id,
                                   self.stack.current_traversal,
                                   sync_point.serialize_input_data(res_data),
                                   self.is_update, {})
        mock_cru.assert_called_once_with(
            self.resource, self.resource.stack.t.id,
            {5, 3}, self.worker.engine_id,
            self.stack, mock.ANY)


@mock.patch.object(check_resource, 'check_stack_complete')
@mock.patch.object(check_resource, 'propagate_check_resource')
@mock.patch.object(check_resource, 'check_resource_cleanup')
@mock.patch.object(check_resource, 'check_resource_update')
class CheckWorkflowCleanupTest(common.HeatTestCase):
    @mock.patch.object(worker_client.WorkerClient, 'check_resource',
                       lambda *_: None)
    def setUp(self):
        super(CheckWorkflowCleanupTest, self).setUp()
        thread_group_mgr = mock.Mock()
        self.worker = worker.WorkerService('host-1',
                                           'topic-1',
                                           'engine_id',
                                           thread_group_mgr)
        self.worker._rpc_client = worker_client.WorkerClient()
        self.ctx = utils.dummy_context()
        tstack = tools.get_stack(
            'check_workflow_create_stack', self.ctx,
            template=tools.string_template_five, convergence=True)
        tstack.converge_stack(tstack.t, action=tstack.CREATE)
        self.stack = stack.Stack.load(self.ctx, stack_id=tstack.id)
        self.stack.thread_group_mgr = tools.DummyThreadGroupManager()
        self.stack.converge_stack(self.stack.t, action=self.stack.DELETE)
        self.resource = self.stack['A']
        self.is_update = False
        self.graph_key = (self.resource.id, self.is_update)

    @mock.patch.object(resource.Resource, 'load')
    @mock.patch.object(stack.Stack, 'time_remaining')
    def test_is_cleanup_traversal(
            self, tr, mock_load, mock_cru, mock_crc, mock_pcr, mock_csc):
        tr.return_value = 317
        mock_load.return_value = self.resource, self.stack, self.stack
        self.worker.check_resource(
            self.ctx, self.resource.id, self.stack.current_traversal, {},
            self.is_update, None)
        self.assertFalse(mock_cru.called)
        mock_crc.assert_called_once_with(
            self.resource, self.resource.stack.t.id,
            self.worker.engine_id,
            tr(), mock.ANY)

    @mock.patch.object(stack.Stack, 'time_remaining')
    def test_is_cleanup_traversal_raise_update_inprogress(
            self, tr, mock_cru, mock_crc, mock_pcr, mock_csc):
        mock_crc.side_effect = exception.UpdateInProgress
        tr.return_value = 317
        self.worker.check_resource(
            self.ctx, self.resource.id, self.stack.current_traversal, {},
            self.is_update, None)
        mock_crc.assert_called_once_with(self.resource,
                                         self.resource.stack.t.id,
                                         self.worker.engine_id,
                                         tr(), mock.ANY)
        self.assertFalse(mock_cru.called)
        self.assertFalse(mock_pcr.called)
        self.assertFalse(mock_csc.called)

    def test_check_resource_does_not_propagate_on_cancelling_cleanup(
            self, mock_cru, mock_crc, mock_pcr, mock_csc):
        # ensure when check_resource is cancelled, the next set of
        # resources are not propagated.
        mock_crc.side_effect = check_resource.CancelOperation
        self.worker.check_resource(self.ctx, self.resource.id,
                                   self.stack.current_traversal,
                                   {}, self.is_update, {})
        self.assertFalse(mock_pcr.called)
        self.assertFalse(mock_csc.called)


class MiscMethodsTest(common.HeatTestCase):
    def setUp(self):
        super(MiscMethodsTest, self).setUp()
        cfg.CONF.set_default('convergence_engine', True)
        self.ctx = utils.dummy_context()
        self.stack = tools.get_stack(
            'check_workflow_create_stack', self.ctx,
            template=tools.attr_cache_template, convergence=True)
        self.stack.converge_stack(self.stack.t)
        self.resource = self.stack['A']

    def test_node_data_ok(self):
        self.resource.action = self.resource.CREATE
        expected_input_data = {'attrs': {(u'flat_dict', u'key2'): 'val2',
                                         (u'flat_dict', u'key3'): 'val3',
                                         (u'nested_dict', u'dict', u'a'): 1,
                                         (u'nested_dict', u'dict', u'b'): 2},
                               'id': mock.ANY,
                               'reference_id': 'A',
                               'name': 'A',
                               'uuid': mock.ANY,
                               'action': mock.ANY,
                               'status': mock.ANY}
        actual_input_data = self.resource.node_data()
        self.assertEqual(expected_input_data, actual_input_data.as_dict())

    def test_node_data_exception(self):
        self.resource.action = self.resource.CREATE
        expected_input_data = {'attrs': {},
                               'id': mock.ANY,
                               'reference_id': 'A',
                               'name': 'A',
                               'uuid': mock.ANY,
                               'action': mock.ANY,
                               'status': mock.ANY}
        self.resource.get_attribute = mock.Mock(
            side_effect=exception.InvalidTemplateAttribute(resource='A',
                                                           key='value'))
        actual_input_data = self.resource.node_data()
        self.assertEqual(expected_input_data, actual_input_data.as_dict())

    @mock.patch.object(sync_point, 'sync')
    def test_check_stack_complete_root(self, mock_sync):
        check_resource.check_stack_complete(
            self.ctx, self.stack, self.stack.current_traversal,
            self.stack['E'].id, self.stack.convergence_dependencies,
            True)
        mock_sync.assert_called_once_with(
            self.ctx, self.stack.id, self.stack.current_traversal, True,
            mock.ANY, mock.ANY, {(self.stack['E'].id, True): None})

    @mock.patch.object(sync_point, 'sync')
    def test_check_stack_complete_child(self, mock_sync):
        check_resource.check_stack_complete(
            self.ctx, self.stack, self.stack.current_traversal,
            self.resource.id, self.stack.convergence_dependencies,
            True)
        self.assertFalse(mock_sync.called)

    @mock.patch.object(dependencies.Dependencies, 'roots')
    @mock.patch.object(stack.Stack, '_persist_state')
    def test_check_stack_complete_persist_called(self, mock_persist_state,
                                                 mock_dep_roots):
        mock_dep_roots.return_value = [(1, True)]
        check_resource.check_stack_complete(
            self.ctx, self.stack, self.stack.current_traversal,
            1, self.stack.convergence_dependencies,
            True)
        self.assertTrue(mock_persist_state.called)

    @mock.patch.object(sync_point, 'sync')
    def test_propagate_check_resource(self, mock_sync):
        check_resource.propagate_check_resource(
            self.ctx, mock.ANY, mock.ANY,
            self.stack.current_traversal, mock.ANY,
            ('A', True), {}, True, None)
        self.assertTrue(mock_sync.called)

    @mock.patch.object(resource.Resource, 'create_convergence')
    @mock.patch.object(resource.Resource, 'update_convergence')
    def test_check_resource_update_init_action(self, mock_update, mock_create):
        self.resource.action = 'INIT'
        check_resource.check_resource_update(
            self.resource, self.resource.stack.t.id, set(), 'engine-id',
            self.stack, None)
        self.assertTrue(mock_create.called)
        self.assertFalse(mock_update.called)

    @mock.patch.object(resource.Resource, 'create_convergence')
    @mock.patch.object(resource.Resource, 'update_convergence')
    def test_check_resource_update_create_action(
            self, mock_update, mock_create):
        self.resource.action = 'CREATE'
        check_resource.check_resource_update(
            self.resource, self.resource.stack.t.id, set(), 'engine-id',
            self.stack, None)
        self.assertFalse(mock_create.called)
        self.assertTrue(mock_update.called)

    @mock.patch.object(resource.Resource, 'create_convergence')
    @mock.patch.object(resource.Resource, 'update_convergence')
    def test_check_resource_update_update_action(
            self, mock_update, mock_create):
        self.resource.action = 'UPDATE'
        check_resource.check_resource_update(
            self.resource, self.resource.stack.t.id, set(), 'engine-id',
            self.stack, None)
        self.assertFalse(mock_create.called)
        self.assertTrue(mock_update.called)

    @mock.patch.object(resource.Resource, 'delete_convergence')
    def test_check_resource_cleanup_delete(self, mock_delete):
        self.resource.current_template_id = 'new-template-id'
        check_resource.check_resource_cleanup(
            self.resource, self.resource.stack.t.id, 'engine-id',
            self.stack.timeout_secs(), None)
        self.assertTrue(mock_delete.called)

    def test_check_message_raises_cancel_exception(self):
        # ensure CancelOperation is raised on receiving
        # rpc_api.THREAD_CANCEL message
        msg_queue = eventlet.queue.LightQueue()
        msg_queue.put_nowait(rpc_api.THREAD_CANCEL)
        self.assertRaises(check_resource.CancelOperation,
                          check_resource._check_for_message, msg_queue)



#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_log import log as logging
from oslo_utils import encodeutils

from heat.common import exception
from heat.common.i18n import _
from heat.engine import attributes
from heat.engine import constraints
from heat.engine import properties
from heat.engine import resource
from heat.engine import support

LOG = logging.getLogger(__name__)


class ManilaShare(resource.Resource):
    """A resource that creates shared mountable file system.

    The resource creates a manila share - shared mountable filesystem that
    can be attached to any client(or clients) that has a network access and
    permission to mount filesystem. Share is a unit of storage with specific
    size that supports pre-defined share protocol and advanced security model
    (access lists, share networks and security services).
    """

    support_status = support.SupportStatus(version='5.0.0')

    _ACCESS_RULE_PROPERTIES = (
        ACCESS_TO, ACCESS_TYPE, ACCESS_LEVEL
    ) = (
        'access_to', 'access_type', 'access_level')

    _SHARE_STATUSES = (
        STATUS_CREATING, STATUS_DELETING, STATUS_ERROR, STATUS_ERROR_DELETING,
        STATUS_AVAILABLE
    ) = (
        'creating', 'deleting', 'error', 'error_deleting',
        'available'
    )

    PROPERTIES = (
        SHARE_PROTOCOL, SIZE, SHARE_SNAPSHOT, NAME, METADATA,
        SHARE_NETWORK, DESCRIPTION, SHARE_TYPE, IS_PUBLIC,
        ACCESS_RULES
    ) = (
        'share_protocol', 'size', 'snapshot', 'name', 'metadata',
        'share_network', 'description', 'share_type', 'is_public',
        'access_rules'
    )

    ATTRIBUTES = (
        AVAILABILITY_ZONE_ATTR, HOST_ATTR, EXPORT_LOCATIONS_ATTR,
        SHARE_SERVER_ID_ATTR, CREATED_AT_ATTR, SHARE_STATUS_ATTR,
        PROJECT_ID_ATTR
    ) = (
        'availability_zone', 'host', 'export_locations',
        'share_server_id', 'created_at', 'status',
        'project_id'
    )

    properties_schema = {
        SHARE_PROTOCOL: properties.Schema(
            properties.Schema.STRING,
            _('Share protocol supported by shared filesystem.'),
            required=True,
            constraints=[constraints.AllowedValues(
                ['NFS', 'CIFS', 'GlusterFS', 'HDFS', 'CEPHFS'])]
        ),
        SIZE: properties.Schema(
            properties.Schema.INTEGER,
            _('Share storage size in GB.'),
            required=True
        ),
        SHARE_SNAPSHOT: properties.Schema(
            properties.Schema.STRING,
            _('Name or ID of shared file system snapshot that '
              'will be restored and created as a new share.'),
            constraints=[constraints.CustomConstraint('manila.share_snapshot')]
        ),
        NAME: properties.Schema(
            properties.Schema.STRING,
            _('Share name.'),
            update_allowed=True
        ),
        METADATA: properties.Schema(
            properties.Schema.MAP,
            _('Metadata key-values defined for share.'),
            update_allowed=True
        ),
        SHARE_NETWORK: properties.Schema(
            properties.Schema.STRING,
            _('Name or ID of shared network defined for shared filesystem.'),
            constraints=[constraints.CustomConstraint('manila.share_network')]
        ),
        DESCRIPTION: properties.Schema(
            properties.Schema.STRING,
            _('Share description.'),
            update_allowed=True
        ),
        SHARE_TYPE: properties.Schema(
            properties.Schema.STRING,
            _('Name or ID of shared filesystem type. Types defines some share '
              'filesystem profiles that will be used for share creation.'),
            constraints=[constraints.CustomConstraint("manila.share_type")]
        ),
        IS_PUBLIC: properties.Schema(
            properties.Schema.BOOLEAN,
            _('Defines if shared filesystem is public or private.'),
            default=False,
            update_allowed=True
        ),
        ACCESS_RULES: properties.Schema(
            properties.Schema.LIST,
            _('A list of access rules that define access from IP to Share.'),
            schema=properties.Schema(
                properties.Schema.MAP,
                schema={
                    ACCESS_TO: properties.Schema(
                        properties.Schema.STRING,
                        _('IP or other address information about guest that '
                          'allowed to access to Share.'),
                        required=True
                    ),
                    ACCESS_TYPE: properties.Schema(
                        properties.Schema.STRING,
                        _('Type of access that should be provided to guest.'),
                        constraints=[constraints.AllowedValues(
                            ['ip', 'user', 'cert', 'cephx'])],
                        required=True
                    ),
                    ACCESS_LEVEL: properties.Schema(
                        properties.Schema.STRING,
                        _('Level of access that need to be provided for '
                          'guest.'),
                        constraints=[constraints.AllowedValues(['ro', 'rw'])]
                    )
                }
            ),
            update_allowed=True,
            default=[]
        )
    }

    attributes_schema = {
        AVAILABILITY_ZONE_ATTR: attributes.Schema(
            _('The availability zone of shared filesystem.'),
            type=attributes.Schema.STRING
        ),
        HOST_ATTR: attributes.Schema(
            _('Share host.'),
            type=attributes.Schema.STRING
        ),
        EXPORT_LOCATIONS_ATTR: attributes.Schema(
            _('Export locations of share.'),
            type=attributes.Schema.LIST
        ),
        SHARE_SERVER_ID_ATTR: attributes.Schema(
            _('ID of server (VM, etc...) on host that is used for '
              'exporting network file-system.'),
            type=attributes.Schema.STRING
        ),
        CREATED_AT_ATTR: attributes.Schema(
            _('Datetime when a share was created.'),
            type=attributes.Schema.STRING
        ),
        SHARE_STATUS_ATTR: attributes.Schema(
            _('Current share status.'),
            type=attributes.Schema.STRING
        ),
        PROJECT_ID_ATTR: attributes.Schema(
            _('Share project ID.'),
            type=attributes.Schema.STRING
        )
    }

    default_client_name = 'manila'

    entity = 'shares'

    def _request_share(self):
        return self.client().shares.get(self.resource_id)

    def _request_export_locations(self):
        # Only return the "path" response parameter, because that is what was
        # returned before API version "2.9" by the shares endpoint
        return [export_location.to_dict()['path']
                for export_location in
                self.client().share_export_locations.list(self.resource_id)]

    def _resolve_attribute(self, name):
        if self.resource_id is None:
            return
        if name == self.EXPORT_LOCATIONS_ATTR:
            attr = self._request_export_locations()
        else:
            attr = getattr(self._request_share(), name)
        return str(attr)

    def handle_create(self):
        # Request IDs of entities from manila
        # if name of the entity defined in template
        share_net_identity = self.properties[self.SHARE_NETWORK]
        if share_net_identity:
            share_net_identity = self.client_plugin().get_share_network(
                share_net_identity).id
        snapshot_identity = self.properties[self.SHARE_SNAPSHOT]
        if snapshot_identity:
            snapshot_identity = self.client_plugin().get_share_snapshot(
                snapshot_identity).id
        share_type_identity = self.properties[self.SHARE_TYPE]
        if share_type_identity:
            share_type_identity = self.client_plugin().get_share_type(
                share_type_identity).id

        share = self.client().shares.create(
            share_proto=self.properties[self.SHARE_PROTOCOL],
            size=self.properties[self.SIZE],
            snapshot_id=snapshot_identity,
            name=self.properties[self.NAME],
            description=self.properties[self.DESCRIPTION],
            metadata=self.properties[self.METADATA],
            share_network=share_net_identity,
            share_type=share_type_identity,
            is_public=self.properties[self.IS_PUBLIC])

        self.resource_id_set(share.id)

    def check_create_complete(self, *args):
        share_status = self._request_share().status
        if share_status == self.STATUS_CREATING:
            return False
        elif share_status == self.STATUS_AVAILABLE:
            LOG.info('Applying access rules to created Share.')
            # apply access rules to created share. please note that it is not
            # possible to define rules for share with share_status = creating
            access_rules = self.properties.get(self.ACCESS_RULES)
            try:
                if access_rules:
                    for rule in access_rules:
                        self.client().shares.allow(
                            share=self.resource_id,
                            access_type=rule.get(self.ACCESS_TYPE),
                            access=rule.get(self.ACCESS_TO),
                            access_level=rule.get(self.ACCESS_LEVEL))
                return True
            except Exception as ex:
                err_msg = encodeutils.exception_to_unicode(ex)
                reason = _(
                    'Error during applying access rules to share "{0}". '
                    'The root cause of the problem is the following: {1}.'
                ).format(self.resource_id, err_msg)
                raise exception.ResourceInError(
                    status_reason=reason, resource_status=share_status)
        elif share_status == self.STATUS_ERROR:
            reason = _('Error during creation of share "{0}"').format(
                self.resource_id)
            raise exception.ResourceInError(status_reason=reason,
                                            resource_status=share_status)
        else:
            reason = _('Unknown share_status during creation of share "{0}"'
                       ).format(self.resource_id)
            raise exception.ResourceUnknownStatus(
                status_reason=reason, resource_status=share_status)

    def check_delete_complete(self, *args):
        if not self.resource_id:
            return True

        try:
            share = self._request_share()
        except Exception as ex:
            self.client_plugin().ignore_not_found(ex)
            return True
        else:
            # when share creation is not finished proceed listening
            if share.status == self.STATUS_DELETING:
                return False
            elif share.status in (self.STATUS_ERROR,
                                  self.STATUS_ERROR_DELETING):
                raise exception.ResourceInError(
                    status_reason=_(
                        'Error during deleting share "{0}".'
                    ).format(self.resource_id),
                    resource_status=share.status)
            else:
                reason = _('Unknown status during deleting share '
                           '"{0}"').format(self.resource_id)
                raise exception.ResourceUnknownStatus(
                    status_reason=reason, resource_status=share.status)

    def handle_check(self):
        share = self._request_share()
        expected_statuses = [self.STATUS_AVAILABLE]
        checks = [{'attr': 'status', 'expected': expected_statuses,
                   'current': share.status}]
        self._verify_check_conditions(checks)

    def handle_update(self, json_snippet, tmpl_diff, prop_diff):
        kwargs = {}
        if self.IS_PUBLIC in prop_diff:
            kwargs['is_public'] = prop_diff.get(self.IS_PUBLIC)
        if self.NAME in prop_diff:
            kwargs['display_name'] = prop_diff.get(self.NAME)
        if self.DESCRIPTION in prop_diff:
            kwargs['display_description'] = prop_diff.get(self.DESCRIPTION)
        if kwargs:
            self.client().shares.update(self.resource_id,
                                        **kwargs)

        if self.METADATA in prop_diff:
            metadata = prop_diff.get(self.METADATA)
            self.client().shares.update_all_metadata(
                self.resource_id, metadata)

        if self.ACCESS_RULES in prop_diff:
            actual_old_rules = []
            for rule in self.client().shares.access_list(self.resource_id):
                old_rule = {
                    self.ACCESS_TO: getattr(rule, self.ACCESS_TO),
                    self.ACCESS_TYPE: getattr(rule, self.ACCESS_TYPE),
                    self.ACCESS_LEVEL: getattr(rule, self.ACCESS_LEVEL)
                }
                if old_rule in prop_diff[self.ACCESS_RULES]:
                    actual_old_rules.append(old_rule)
                else:
                    self.client().shares.deny(share=self.resource_id,
                                              id=rule.id)
            for rule in prop_diff[self.ACCESS_RULES]:
                if rule not in actual_old_rules:
                    self.client().shares.allow(
                        share=self.resource_id,
                        access_type=rule.get(self.ACCESS_TYPE),
                        access=rule.get(self.ACCESS_TO),
                        access_level=rule.get(self.ACCESS_LEVEL)
                    )

    def parse_live_resource_data(self, resource_properties, resource_data):
        result = super(ManilaShare, self).parse_live_resource_data(
            resource_properties, resource_data)

        rules = self.client().shares.access_list(self.resource_id)
        result[self.ACCESS_RULES] = []
        for rule in rules:
            result[self.ACCESS_RULES].append(
                {(k, v) for (k, v) in rule.items()
                 if k in self._ACCESS_RULE_PROPERTIES})
        return result


def resource_mapping():
    return {'OS::Manila::Share': ManilaShare}



#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

SPOOL_TEMPLATE = '''
heat_template_version: 2015-04-30
description: Template to test subnetpool Neutron resource
resources:
  sub_pool:
    type: OS::Neutron::SubnetPool
    properties:
      name: the_sp
      prefixes:
        - 10.1.0.0/16
      address_scope: test
      default_quota: 2
      default_prefixlen: 28
      min_prefixlen: 8
      max_prefixlen: 32
      is_default: False
      tenant_id: c1210485b2424d48804aad5d39c61b8f
      shared: False
'''

SPOOL_MINIMAL_TEMPLATE = '''
heat_template_version: 2015-04-30
description: Template to test subnetpool Neutron resource
resources:
  sub_pool:
    type: OS::Neutron::SubnetPool
    properties:
      prefixes:
        - 10.0.0.0/16
        - 10.1.0.0/16
'''

RBAC_TEMPLATE = '''
heat_template_version: 2016-04-08
description: Template to test rbac-policy Neutron resource
resources:
  rbac:
    type: OS::Neutron::RBACPolicy
    properties:
      object_type: network
      target_tenant: d1dbbed707e5469da9cd4fdd618e9706
      action: access_as_shared
      object_id: 9ba4c03a-dbd5-4836-b651-defa595796ba
'''

RBAC_REFERENCE_TEMPLATE = '''
heat_template_version: 2016-04-08
description: Template to test rbac-policy Neutron resource
resources:
  rbac:
    type: OS::Neutron::RBACPolicy
    properties:
      object_type: network
      target_tenant: d1dbbed707e5469da9cd4fdd618e9706
      action: access_as_shared
      object_id: {get_resource: my_net}
  my_net:
    type: OS::Neutron::Net
'''

LB_TEMPLATE = '''
heat_template_version: 2016-04-08
description: Create a loadbalancer
resources:
  lb:
    type: OS::Neutron::LBaaS::LoadBalancer
    properties:
      name: my_lb
      description: my loadbalancer
      vip_address: 10.0.0.4
      vip_subnet: sub123
      provider: octavia
      tenant_id: 1234
      admin_state_up: True
'''

LISTENER_TEMPLATE = '''
heat_template_version: 2016-04-08
description: Create a listener
resources:
  listener:
    type: OS::Neutron::LBaaS::Listener
    properties:
      protocol_port: 80
      protocol: TCP
      loadbalancer: 123
      default_pool: my_pool
      name: my_listener
      description: my listener
      admin_state_up: True
      default_tls_container_ref: ref
      sni_container_refs:
        - ref1
        - ref2
      connection_limit: -1
      tenant_id: 1234
'''

POOL_TEMPLATE = '''
heat_template_version: 2016-04-08
description: Create a pool
resources:
  pool:
    type: OS::Neutron::LBaaS::Pool
    properties:
      name: my_pool
      description: my pool
      session_persistence:
        type: HTTP_COOKIE
      lb_algorithm: ROUND_ROBIN
      loadbalancer: my_lb
      listener: 123
      protocol: HTTP
      admin_state_up: True
'''

MEMBER_TEMPLATE = '''
heat_template_version: 2016-04-08
description: Create a pool member
resources:
  member:
    type: OS::Neutron::LBaaS::PoolMember
    properties:
      pool: 123
      address: 1.2.3.4
      protocol_port: 80
      weight: 1
      subnet: sub123
      admin_state_up: True
'''

MONITOR_TEMPLATE = '''
heat_template_version: 2016-04-08
description: Create a health monitor
resources:
  monitor:
    type: OS::Neutron::LBaaS::HealthMonitor
    properties:
      admin_state_up: True
      delay: 3
      expected_codes: 200-202
      http_method: HEAD
      max_retries: 5
      pool: 123
      timeout: 10
      type: HTTP
      url_path: /health
'''

SECURITY_GROUP_RULE_TEMPLATE = '''
heat_template_version: 2016-10-14
resources:
  security_group_rule:
    type: OS::Neutron::SecurityGroupRule
    properties:
      security_group: 123
      description: test description
      remote_group: 123
      protocol: tcp
      port_range_min: 100
'''

L7POLICY_TEMPLATE = '''
heat_template_version: 2016-04-08
description: Template to test L7Policy Neutron resource
resources:
  l7policy:
    type: OS::Neutron::LBaaS::L7Policy
    properties:
      admin_state_up: True
      name: test_l7policy
      description: test l7policy resource
      action: REDIRECT_TO_URL
      redirect_url: http://www.mirantis.com
      listener: 123
      position: 1
'''

L7RULE_TEMPLATE = '''
heat_template_version: 2016-04-08
description: Template to test L7Rule Neutron resource
resources:
  l7rule:
    type: OS::Neutron::LBaaS::L7Rule
    properties:
      admin_state_up: True
      l7policy: 123
      type: HEADER
      compare_type: ENDS_WITH
      key: test_key
      value: test_value
      invert: False
'''

SEGMENT_TEMPLATE = '''
heat_template_version: pike
description: Template to test Segment
resources:
  segment:
    type: OS::Neutron::Segment
    properties:
      network: private
      network_type: vxlan
      segmentation_id: 101
'''



#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy

from heat.common import exception
from heat.common.i18n import _
from heat.engine import scheduler


def reconfigure_loadbalancers(load_balancers, id_list):
    """Notify the LoadBalancer to reload its config.

    This must be done after activation (instance in ACTIVE state), otherwise
    the instances' IP addresses may not be available.
    """
    for lb in load_balancers:
        existing_defn = lb.frozen_definition()
        props = copy.copy(existing_defn.properties(lb.properties_schema,
                                                   lb.context).data)
        if 'Instances' in lb.properties_schema:
            props['Instances'] = id_list
        elif 'members' in lb.properties_schema:
            props['members'] = id_list
        else:
            raise exception.Error(
                _("Unsupported resource '%s' in LoadBalancerNames") % lb.name)

        lb_defn = existing_defn.freeze(properties=props)

        scheduler.TaskRunner(lb.update, lb_defn)()



# -*- coding: utf-8 -*-
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import os
from unittest import mock

from keystoneauth1 import loading as ks_loading
from oslo_config import cfg
from oslo_config import fixture as config_fixture
from oslo_middleware import request_id
from oslo_policy import opts as policy_opts
from oslo_utils import importutils
import webob

from heat.common import context
from heat.common import exception
from heat.tests import common

policy_path = os.path.dirname(os.path.realpath(__file__)) + "/policy/"


class TestRequestContext(common.HeatTestCase):

    def setUp(self):
        self.ctx = {'username': 'mick',
                    'trustor_user_id': None,
                    'auth_token': '123',
                    'auth_token_info': {'123info': 'woop'},
                    'is_admin': False,
                    'user': 'mick',
                    'password': 'foo',
                    'trust_id': None,
                    'global_request_id': None,
                    'show_deleted': False,
                    'roles': ['arole', 'notadmin'],
                    'tenant_id': '456tenant',
                    'user_id': 'fooUser',
                    'tenant': u'\u5218\u80dc',
                    'auth_url': 'http://xyz',
                    'aws_creds': 'blah',
                    'region_name': 'RegionOne',
                    'user_identity': 'fooUser 456tenant',
                    'user_domain': None,
                    'project_domain': None}

        super(TestRequestContext, self).setUp()

    def test_request_context_init(self):
        ctx = context.RequestContext(
            auth_token=self.ctx.get('auth_token'),
            username=self.ctx.get('username'),
            password=self.ctx.get('password'),
            aws_creds=self.ctx.get('aws_creds'),
            project_name=self.ctx.get('tenant'),
            tenant=self.ctx.get('tenant_id'),
            user=self.ctx.get('user_id'),
            auth_url=self.ctx.get('auth_url'),
            roles=self.ctx.get('roles'),
            show_deleted=self.ctx.get('show_deleted'),
            is_admin=self.ctx.get('is_admin'),
            auth_token_info=self.ctx.get('auth_token_info'),
            trustor_user_id=self.ctx.get('trustor_user_id'),
            trust_id=self.ctx.get('trust_id'),
            region_name=self.ctx.get('region_name'),
            user_domain_id=self.ctx.get('user_domain'),
            project_domain_id=self.ctx.get('project_domain'))
        ctx_dict = ctx.to_dict()
        del ctx_dict['request_id']
        del ctx_dict['project_id']
        del ctx_dict['project_name']
        self.assertEqual(self.ctx, ctx_dict)

    def test_request_context_to_dict_unicode(self):

        ctx_origin = {'username': 'mick',
                      'trustor_user_id': None,
                      'auth_token': '123',
                      'auth_token_info': {'123info': 'woop'},
                      'is_admin': False,
                      'user': 'mick',
                      'password': 'foo',
                      'trust_id': None,
                      'global_request_id': None,
                      'show_deleted': False,
                      'roles': ['arole', 'notadmin'],
                      'tenant_id': '456tenant',
                      'project_id': '456tenant',
                      'user_id': u'Gāo',
                      'tenant': u'\u5218\u80dc',
                      'project_name': u'\u5218\u80dc',
                      'auth_url': 'http://xyz',
                      'aws_creds': 'blah',
                      'region_name': 'RegionOne',
                      'user_identity': u'Gāo 456tenant',
                      'user_domain': None,
                      'project_domain': None}

        ctx = context.RequestContext(
            auth_token=ctx_origin.get('auth_token'),
            username=ctx_origin.get('username'),
            password=ctx_origin.get('password'),
            aws_creds=ctx_origin.get('aws_creds'),
            project_name=ctx_origin.get('tenant'),
            tenant=ctx_origin.get('tenant_id'),
            user=ctx_origin.get('user_id'),
            auth_url=ctx_origin.get('auth_url'),
            roles=ctx_origin.get('roles'),
            show_deleted=ctx_origin.get('show_deleted'),
            is_admin=ctx_origin.get('is_admin'),
            auth_token_info=ctx_origin.get('auth_token_info'),
            trustor_user_id=ctx_origin.get('trustor_user_id'),
            trust_id=ctx_origin.get('trust_id'),
            region_name=ctx_origin.get('region_name'),
            user_domain_id=ctx_origin.get('user_domain'),
            project_domain_id=ctx_origin.get('project_domain'))
        ctx_dict = ctx.to_dict()
        del(ctx_dict['request_id'])
        self.assertEqual(ctx_origin, ctx_dict)

    def test_request_context_from_dict(self):
        ctx = context.RequestContext.from_dict(self.ctx)
        ctx_dict = ctx.to_dict()
        del ctx_dict['request_id']
        del ctx_dict['project_id']
        del ctx_dict['project_name']
        self.assertEqual(self.ctx, ctx_dict)

    def test_request_context_update(self):
        ctx = context.RequestContext.from_dict(self.ctx)

        for k in self.ctx:
            if (k == 'user_identity' or
                    k == 'user_domain_id' or
                    k == 'project_domain_id'):
                continue

            # these values are different between attribute and context
            if k == 'tenant' or k == 'user':
                continue

            self.assertEqual(self.ctx.get(k), ctx.to_dict().get(k))
            override = '%s_override' % k
            setattr(ctx, k, override)
            self.assertEqual(override, ctx.to_dict().get(k))

    def test_get_admin_context(self):
        ctx = context.get_admin_context()
        self.assertTrue(ctx.is_admin)
        self.assertFalse(ctx.show_deleted)

    def test_get_admin_context_show_deleted(self):
        ctx = context.get_admin_context(show_deleted=True)
        self.assertTrue(ctx.is_admin)
        self.assertTrue(ctx.show_deleted)

    def test_admin_context_policy_true(self):
        policy_check = 'heat.common.policy.Enforcer.check_is_admin'
        with mock.patch(policy_check) as pc:
            pc.return_value = True
            ctx = context.RequestContext(roles=['admin'])
            self.assertTrue(ctx.is_admin)

    def test_admin_context_policy_false(self):
        policy_check = 'heat.common.policy.Enforcer.check_is_admin'
        with mock.patch(policy_check) as pc:
            pc.return_value = False
            ctx = context.RequestContext(roles=['notadmin'])
            self.assertFalse(ctx.is_admin)

    def test_keystone_v3_endpoint_in_context(self):
        """Ensure that the context is the preferred source for the auth_uri."""
        cfg.CONF.set_override('auth_uri', 'http://xyz',
                              group='clients_keystone')
        policy_check = 'heat.common.policy.Enforcer.check_is_admin'
        with mock.patch(policy_check) as pc:
            pc.return_value = False
            ctx = context.RequestContext(
                auth_url='http://example.com:5000/v2.0')
            self.assertEqual(ctx.keystone_v3_endpoint,
                             'http://example.com:5000/v3')

    def test_keystone_v3_endpoint_in_clients_keystone_config(self):
        """Ensure that the [clients_keystone] section is the preferred source.

        Ensure that the [clients_keystone] section of the configuration is
        the preferred source when the context does not have the auth_uri.
        """
        cfg.CONF.set_override('auth_uri', 'http://xyz',
                              group='clients_keystone')
        policy_check = 'heat.common.policy.Enforcer.check_is_admin'
        with mock.patch(policy_check) as pc:
            pc.return_value = False
            with mock.patch('keystoneauth1.discover.Discover') as discover:
                class MockDiscover(object):
                    def url_for(self, endpoint):
                        return 'http://xyz/v3'
                discover.return_value = MockDiscover()

                ctx = context.RequestContext(auth_url=None)
                self.assertEqual(ctx.keystone_v3_endpoint, 'http://xyz/v3')

    def test_keystone_v3_endpoint_in_keystone_authtoken_config(self):
        """Ensure that the [keystone_authtoken] section is used.

        Ensure that the [keystone_authtoken] section of the configuration
        is used when the auth_uri is not defined in the context or the
        [clients_keystone] section.
        """
        importutils.import_module('keystonemiddleware.auth_token')
        try:
            cfg.CONF.set_override('www_authenticate_uri', 'http://abc/v2.0',
                                  group='keystone_authtoken')
        except cfg.NoSuchOptError:
            cfg.CONF.set_override('auth_uri', 'http://abc/v2.0',
                                  group='keystone_authtoken')
        policy_check = 'heat.common.policy.Enforcer.check_is_admin'
        with mock.patch(policy_check) as pc:
            pc.return_value = False
            ctx = context.RequestContext(auth_url=None)
            self.assertEqual(ctx.keystone_v3_endpoint, 'http://abc/v3')

    def test_keystone_v3_endpoint_not_set_in_config(self):
        """Ensure an exception is raised when the auth_uri cannot be obtained.

        Ensure an exception is raised when the auth_uri cannot be obtained
        from any source.
        """
        policy_check = 'heat.common.policy.Enforcer.check_is_admin'
        with mock.patch(policy_check) as pc:
            pc.return_value = False
            ctx = context.RequestContext(auth_url=None)
            self.assertRaises(exception.AuthorizationFailure, getattr, ctx,
                              'keystone_v3_endpoint')

    def test_get_trust_context_auth_plugin_unauthorized(self):
        self.ctx['trust_id'] = 'trust_id'
        ctx = context.RequestContext.from_dict(self.ctx)
        self.patchobject(ks_loading, 'load_auth_from_conf_options',
                         return_value=None)
        self.assertRaises(exception.AuthorizationFailure, getattr,
                          ctx, 'auth_plugin')

    def test_cache(self):
        ctx = context.RequestContext.from_dict(self.ctx)

        class Class1(object):
            pass

        class Class2(object):
            pass

        self.assertEqual(0, len(ctx._object_cache))

        cache1 = ctx.cache(Class1)
        self.assertIsInstance(cache1, Class1)
        self.assertEqual(1, len(ctx._object_cache))

        cache1a = ctx.cache(Class1)
        self.assertEqual(cache1, cache1a)
        self.assertEqual(1, len(ctx._object_cache))

        cache2 = ctx.cache(Class2)
        self.assertIsInstance(cache2, Class2)
        self.assertEqual(2, len(ctx._object_cache))


class RequestContextMiddlewareTest(common.HeatTestCase):

    scenarios = [(
        'empty_headers',
        dict(
            environ=None,
            headers={},
            context_dict={
                'auth_token': None,
                'auth_token_info': None,
                'auth_url': None,
                'aws_creds': None,
                'is_admin': False,
                'password': None,
                'roles': [],
                'show_deleted': False,
                'tenant': None,
                'tenant_id': None,
                'trust_id': None,
                'trustor_user_id': None,
                'user': None,
                'user_id': None,
                'username': None
            })
    ), (
        'username_password',
        dict(
            environ=None,
            headers={
                'X-Auth-User': 'my_username',
                'X-Auth-Key': 'my_password',
                'X-Auth-EC2-Creds': '{"ec2Credentials": {}}',
                'X-User-Id': '7a87ff18-31c6-45ce-a186-ec7987f488c3',
                'X-Auth-Token': 'atoken',
                'X-Project-Name': 'my_tenant',
                'X-Project-Id': 'db6808c8-62d0-4d92-898c-d644a6af20e9',
                'X-Auth-Url': 'http://192.0.2.1:5000/v1',
                'X-Roles': 'role1,role2,role3'
            },
            context_dict={
                'auth_token': 'atoken',
                'auth_url': 'http://192.0.2.1:5000/v1',
                'aws_creds': None,
                'is_admin': False,
                'password': 'my_password',
                'roles': ['role1', 'role2', 'role3'],
                'show_deleted': False,
                'tenant': 'my_tenant',
                'tenant_id': 'db6808c8-62d0-4d92-898c-d644a6af20e9',
                'trust_id': None,
                'trustor_user_id': None,
                'user': 'my_username',
                'user_id': '7a87ff18-31c6-45ce-a186-ec7987f488c3',
                'username': 'my_username'
            })
    ), (
        'aws_creds',
        dict(
            environ=None,
            headers={
                'X-Auth-EC2-Creds': '{"ec2Credentials": {}}',
                'X-User-Id': '7a87ff18-31c6-45ce-a186-ec7987f488c3',
                'X-Auth-Token': 'atoken',
                'X-Project-Name': 'my_tenant',
                'X-Project-Id': 'db6808c8-62d0-4d92-898c-d644a6af20e9',
                'X-Auth-Url': 'http://192.0.2.1:5000/v1',
                'X-Roles': 'role1,role2,role3',
            },
            context_dict={
                'auth_token': 'atoken',
                'auth_url': 'http://192.0.2.1:5000/v1',
                'aws_creds': '{"ec2Credentials": {}}',
                'is_admin': False,
                'password': None,
                'roles': ['role1', 'role2', 'role3'],
                'show_deleted': False,
                'tenant': 'my_tenant',
                'tenant_id': 'db6808c8-62d0-4d92-898c-d644a6af20e9',
                'trust_id': None,
                'trustor_user_id': None,
                'user': None,
                'user_id': '7a87ff18-31c6-45ce-a186-ec7987f488c3',
                'username': None
            })
    ), (
        'token_creds',
        dict(
            environ={'keystone.token_info': {'info': 123}},
            headers={
                'X-User-Id': '7a87ff18-31c6-45ce-a186-ec7987f488c3',
                'X-Auth-Token': 'atoken2',
                'X-Project-Name': 'my_tenant2',
                'X-Project-Id': 'bb9108c8-62d0-4d92-898c-d644a6af20e9',
                'X-Auth-Url': 'http://192.0.2.1:5000/v1',
                'X-Roles': 'role1,role2,role3',
            },
            context_dict={
                'auth_token': 'atoken2',
                'auth_token_info': {'info': 123},
                'auth_url': 'http://192.0.2.1:5000/v1',
                'aws_creds': None,
                'is_admin': False,
                'password': None,
                'roles': ['role1', 'role2', 'role3'],
                'show_deleted': False,
                'tenant': 'my_tenant2',
                'tenant_id': 'bb9108c8-62d0-4d92-898c-d644a6af20e9',
                'trust_id': None,
                'trustor_user_id': None,
                'user': None,
                'user_id': '7a87ff18-31c6-45ce-a186-ec7987f488c3',
                'username': None
            })
    )]

    def setUp(self):
        super(RequestContextMiddlewareTest, self).setUp()
        self.fixture = self.useFixture(config_fixture.Config())
        self.fixture.conf(args=['--config-dir', policy_path])
        policy_opts.set_defaults(cfg.CONF, 'check_admin.json')

    def test_context_middleware(self):

        middleware = context.ContextMiddleware(None, None)
        request = webob.Request.blank('/stacks', headers=self.headers,
                                      environ=self.environ)

        self.assertIsNone(middleware.process_request(request))
        ctx = request.context.to_dict()
        for k, v in self.context_dict.items():
            self.assertEqual(v, ctx[k], 'Key %s values do not match' % k)
        self.assertIsNotNone(ctx.get('request_id'))

    def test_context_middleware_with_requestid(self):

        middleware = context.ContextMiddleware(None, None)
        request = webob.Request.blank('/stacks', headers=self.headers,
                                      environ=self.environ)
        req_id = 'req-5a63f0d7-1b69-447b-b621-4ea87cc7186d'
        request.environ[request_id.ENV_REQUEST_ID] = req_id

        self.assertIsNone(middleware.process_request(request))
        ctx = request.context.to_dict()
        for k, v in self.context_dict.items():
            self.assertEqual(v, ctx[k], 'Key %s values do not match' % k)
        self.assertEqual(
            ctx.get('request_id'), req_id,
            'Key request_id values do not match')



#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy
from unittest import mock

from heat.engine.clients.os.keystone import fake_keystoneclient as fake_ks
from heat.engine import constraints
from heat.engine import properties
from heat.engine import resource
from heat.engine.resources.openstack.keystone import endpoint
from heat.engine import stack
from heat.engine import template
from heat.tests import common
from heat.tests import utils

keystone_endpoint_template = {
    'heat_template_version': '2015-04-30',
    'resources': {
        'test_endpoint': {
            'type': 'OS::Keystone::Endpoint',
            'properties': {
                'service': 'heat',
                'region': 'RegionOne',
                'interface': 'public',
                'url': 'http://127.0.0.1:8004/v1/tenant-id',
                'name': 'endpoint_foo',
                'enabled': False
            }
        }
    }
}


class KeystoneEndpointTest(common.HeatTestCase):
    def setUp(self):
        super(KeystoneEndpointTest, self).setUp()

        self.ctx = utils.dummy_context()

        # Mock client
        self.keystoneclient = mock.Mock()
        self.patchobject(resource.Resource, 'client',
                         return_value=fake_ks.FakeKeystoneClient(
                             client=self.keystoneclient))
        self.endpoints = self.keystoneclient.endpoints

        # Mock client plugin
        self.keystone_client_plugin = mock.MagicMock()

    def _get_mock_endpoint(self):
        value = mock.MagicMock()
        value.id = '477e8273-60a7-4c41-b683-fdb0bc7cd152'

        return value

    def _setup_endpoint_resource(self, stack_name, use_default=False):
        tmpl_data = copy.deepcopy(keystone_endpoint_template)
        if use_default:
            props = tmpl_data['resources']['test_endpoint']['properties']
            del props['name']
            del props['enabled']

        test_stack = stack.Stack(
            self.ctx, stack_name,
            template.Template(tmpl_data)
        )

        r_endpoint = test_stack['test_endpoint']
        r_endpoint.client = mock.MagicMock()
        r_endpoint.client.return_value = self.keystoneclient
        r_endpoint.client_plugin = mock.MagicMock()
        r_endpoint.client_plugin.return_value = self.keystone_client_plugin

        return r_endpoint

    def test_endpoint_handle_create(self):
        rsrc = self._setup_endpoint_resource('test_endpoint_create')
        mock_endpoint = self._get_mock_endpoint()
        self.endpoints.create.return_value = mock_endpoint

        # validate the properties
        self.assertEqual(
            'heat', rsrc.properties.get(endpoint.KeystoneEndpoint.SERVICE))
        self.assertEqual(
            'public',
            rsrc.properties.get(endpoint.KeystoneEndpoint.INTERFACE))
        self.assertEqual(
            'RegionOne',
            rsrc.properties.get(endpoint.KeystoneEndpoint.REGION))
        self.assertEqual(
            'http://127.0.0.1:8004/v1/tenant-id',
            rsrc.properties.get(endpoint.KeystoneEndpoint.SERVICE_URL))
        self.assertEqual(
            'endpoint_foo',
            rsrc.properties.get(endpoint.KeystoneEndpoint.NAME))
        self.assertFalse(rsrc.properties.get(
            endpoint.KeystoneEndpoint.ENABLED))

        rsrc.handle_create()

        # validate endpoint creation
        self.endpoints.create.assert_called_once_with(
            service='heat',
            url='http://127.0.0.1:8004/v1/tenant-id',
            interface='public',
            region='RegionOne',
            name='endpoint_foo',
            enabled=False)

        # validate physical resource id
        self.assertEqual(mock_endpoint.id, rsrc.resource_id)

    def test_endpoint_handle_create_default(self):
        rsrc = self._setup_endpoint_resource('test_create_with_defaults',
                                             use_default=True)
        mock_endpoint = self._get_mock_endpoint()
        self.endpoints.create.return_value = mock_endpoint

        rsrc.physical_resource_name = mock.MagicMock()
        rsrc.physical_resource_name.return_value = 'stack_endpoint_foo'

        # validate the properties
        self.assertIsNone(
            rsrc.properties.get(endpoint.KeystoneEndpoint.NAME))
        self.assertTrue(rsrc.properties.get(
            endpoint.KeystoneEndpoint.ENABLED))

        rsrc.handle_create()

        # validate endpoints creation with physical resource name
        # and with enabled(default is True)
        self.endpoints.create.assert_called_once_with(
            service='heat',
            url='http://127.0.0.1:8004/v1/tenant-id',
            interface='public',
            region='RegionOne',
            name='stack_endpoint_foo',
            enabled=True)

    def test_endpoint_handle_update(self):
        rsrc = self._setup_endpoint_resource('test_endpoint_update')
        rsrc.resource_id = '477e8273-60a7-4c41-b683-fdb0bc7cd151'

        prop_diff = {endpoint.KeystoneEndpoint.REGION: 'RegionTwo',
                     endpoint.KeystoneEndpoint.INTERFACE: 'internal',
                     endpoint.KeystoneEndpoint.SERVICE: 'updated_id',
                     endpoint.KeystoneEndpoint.SERVICE_URL:
                         'http://127.0.0.1:8004/v2/tenant-id',
                     endpoint.KeystoneEndpoint.NAME:
                         'endpoint_foo_updated',
                     endpoint.KeystoneEndpoint.ENABLED: True}

        rsrc.handle_update(json_snippet=None,
                           tmpl_diff=None,
                           prop_diff=prop_diff)

        self.endpoints.update.assert_called_once_with(
            endpoint=rsrc.resource_id,
            region=prop_diff[endpoint.KeystoneEndpoint.REGION],
            interface=prop_diff[endpoint.KeystoneEndpoint.INTERFACE],
            service=prop_diff[endpoint.KeystoneEndpoint.SERVICE],
            url=prop_diff[endpoint.KeystoneEndpoint.SERVICE_URL],
            name=prop_diff[endpoint.KeystoneEndpoint.NAME],
            enabled=prop_diff[endpoint.KeystoneEndpoint.ENABLED]
        )

    def test_endpoint_handle_update_default(self):
        rsrc = self._setup_endpoint_resource('test_endpoint_update_default')
        rsrc.resource_id = '477e8273-60a7-4c41-b683-fdb0bc7cd151'
        rsrc.physical_resource_name = mock.MagicMock()
        rsrc.physical_resource_name.return_value = 'stack_endpoint_foo'

        # Name is reset to None, so default to physical resource name
        prop_diff = {endpoint.KeystoneEndpoint.NAME: None}

        rsrc.handle_update(json_snippet=None,
                           tmpl_diff=None,
                           prop_diff=prop_diff)

        # validate default name to physical resource name
        self.endpoints.update.assert_called_once_with(
            endpoint=rsrc.resource_id,
            region=None,
            interface=None,
            service=None,
            url=None,
            name='stack_endpoint_foo',
            enabled=None
        )

    def test_endpoint_handle_update_only_enabled(self):
        rsrc = self._setup_endpoint_resource('test_endpoint_update_enabled')
        rsrc.resource_id = '477e8273-60a7-4c41-b683-fdb0bc7cd151'

        prop_diff = {endpoint.KeystoneEndpoint.ENABLED: True}

        rsrc.handle_update(json_snippet=None,
                           tmpl_diff=None,
                           prop_diff=prop_diff)

        self.endpoints.update.assert_called_once_with(
            endpoint=rsrc.resource_id,
            region=None,
            interface=None,
            service=None,
            url=None,
            name=None,
            enabled=prop_diff[endpoint.KeystoneEndpoint.ENABLED]
        )

    def test_properties_title(self):
        property_title_map = {
            endpoint.KeystoneEndpoint.SERVICE: 'service',
            endpoint.KeystoneEndpoint.REGION: 'region',
            endpoint.KeystoneEndpoint.INTERFACE: 'interface',
            endpoint.KeystoneEndpoint.SERVICE_URL: 'url',
            endpoint.KeystoneEndpoint.NAME: 'name',
            endpoint.KeystoneEndpoint.ENABLED: 'enabled'
        }

        for actual_title, expected_title in property_title_map.items():
            self.assertEqual(
                expected_title,
                actual_title,
                'KeystoneEndpoint PROPERTIES(%s) title modified.' %
                actual_title)

    def test_property_service_validate_schema(self):
        schema = (endpoint.KeystoneEndpoint.properties_schema[
                  endpoint.KeystoneEndpoint.SERVICE])
        self.assertTrue(
            schema.update_allowed,
            'update_allowed for property %s is modified' %
            endpoint.KeystoneEndpoint.SERVICE)

        self.assertTrue(
            schema.required,
            'required for property %s is modified' %
            endpoint.KeystoneEndpoint.SERVICE)

        self.assertEqual(properties.Schema.STRING,
                         schema.type,
                         'type for property %s is modified' %
                         endpoint.KeystoneEndpoint.SERVICE)

        self.assertEqual('Name or Id of keystone service.',
                         schema.description,
                         'description for property %s is modified' %
                         endpoint.KeystoneEndpoint.SERVICE)

        # Make sure, SERVICE is of keystone.service custom constrain type
        self.assertEqual(1, len(schema.constraints))
        keystone_service_constrain = schema.constraints[0]
        self.assertIsInstance(keystone_service_constrain,
                              constraints.CustomConstraint)
        self.assertEqual('keystone.service',
                         keystone_service_constrain.name)

    def test_property_region_validate_schema(self):
        schema = (endpoint.KeystoneEndpoint.properties_schema[
                  endpoint.KeystoneEndpoint.REGION])
        self.assertTrue(
            schema.update_allowed,
            'update_allowed for property %s is modified' %
            endpoint.KeystoneEndpoint.REGION)

        self.assertEqual(properties.Schema.STRING,
                         schema.type,
                         'type for property %s is modified' %
                         endpoint.KeystoneEndpoint.REGION)

        self.assertEqual('Name or Id of keystone region.',
                         schema.description,
                         'description for property %s is modified' %
                         endpoint.KeystoneEndpoint.REGION)

        # Make sure, REGION is of keystone.region custom constraint type
        self.assertEqual(1, len(schema.constraints))
        keystone_region_constraint = schema.constraints[0]
        self.assertIsInstance(keystone_region_constraint,
                              constraints.CustomConstraint)
        self.assertEqual('keystone.region',
                         keystone_region_constraint.name)

    def test_property_interface_validate_schema(self):
        schema = (endpoint.KeystoneEndpoint.properties_schema[
                  endpoint.KeystoneEndpoint.INTERFACE])
        self.assertTrue(
            schema.update_allowed,
            'update_allowed for property %s is modified' %
            endpoint.KeystoneEndpoint.INTERFACE)

        self.assertTrue(
            schema.required,
            'required for property %s is modified' %
            endpoint.KeystoneEndpoint.INTERFACE)

        self.assertEqual(properties.Schema.STRING,
                         schema.type,
                         'type for property %s is modified' %
                         endpoint.KeystoneEndpoint.INTERFACE)

        self.assertEqual('Interface type of keystone service endpoint.',
                         schema.description,
                         'description for property %s is modified' %
                         endpoint.KeystoneEndpoint.INTERFACE)

        # Make sure INTERFACE valid constrains
        self.assertEqual(1, len(schema.constraints))
        allowed_constrain = schema.constraints[0]
        self.assertIsInstance(allowed_constrain,
                              constraints.AllowedValues)
        self.assertEqual(('public', 'internal', 'admin'),
                         allowed_constrain.allowed)

    def test_property_service_url_validate_schema(self):
        schema = (endpoint.KeystoneEndpoint.properties_schema[
                  endpoint.KeystoneEndpoint.SERVICE_URL])
        self.assertTrue(
            schema.update_allowed,
            'update_allowed for property %s is modified' %
            endpoint.KeystoneEndpoint.SERVICE_URL)

        self.assertTrue(
            schema.required,
            'required for property %s is modified' %
            endpoint.KeystoneEndpoint.SERVICE_URL)

        self.assertEqual(properties.Schema.STRING,
                         schema.type,
                         'type for property %s is modified' %
                         endpoint.KeystoneEndpoint.SERVICE_URL)

        self.assertEqual('URL of keystone service endpoint.',
                         schema.description,
                         'description for property %s is modified' %
                         endpoint.KeystoneEndpoint.SERVICE_URL)

    def test_property_name_validate_schema(self):
        schema = (endpoint.KeystoneEndpoint.properties_schema[
                  endpoint.KeystoneEndpoint.NAME])
        self.assertTrue(
            schema.update_allowed,
            'update_allowed for property %s is modified' %
            endpoint.KeystoneEndpoint.NAME)

        self.assertEqual(properties.Schema.STRING,
                         schema.type,
                         'type for property %s is modified' %
                         endpoint.KeystoneEndpoint.NAME)

        self.assertEqual('Name of keystone endpoint.',
                         schema.description,
                         'description for property %s is modified' %
                         endpoint.KeystoneEndpoint.NAME)

    def test_show_resource(self):
        rsrc = self._setup_endpoint_resource('test_show_resource')
        mock_endpoint = mock.Mock()
        mock_endpoint.to_dict.return_value = {'attr': 'val'}
        self.endpoints.get.return_value = mock_endpoint
        attrs = rsrc._show_resource()
        self.assertEqual({'attr': 'val'}, attrs)

    def test_get_live_state(self):
        rsrc = self._setup_endpoint_resource('test_get_live_state')
        mock_endpoint = mock.Mock()
        mock_endpoint.to_dict.return_value = {
            'region_id': 'RegionOne',
            'links': {'self': 'some_link'},
            'url': 'http://127.0.0.1:8004/v1/1234',
            'region': 'RegionOne',
            'enabled': True,
            'interface': 'admin',
            'service_id': '934f10ea63c24d82a8d9370cc0a1cb3b',
            'id': '7f1944ae8c524e2799119b5f2dcf9781',
            'name': 'fake'}
        self.endpoints.get.return_value = mock_endpoint

        reality = rsrc.get_live_state(rsrc.properties)
        expected = {
            'region': 'RegionOne',
            'enabled': True,
            'interface': 'admin',
            'service': '934f10ea63c24d82a8d9370cc0a1cb3b',
            'name': 'fake',
            'url': 'http://127.0.0.1:8004/v1/1234'
        }

        self.assertEqual(expected, reality)



#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from heat.common.i18n import _
from heat.engine import attributes
from heat.engine import constraints
from heat.engine import properties
from heat.engine.resources.openstack.neutron import neutron
from heat.engine import support


class MeteringLabel(neutron.NeutronResource):
    """A resource for creating neutron metering label.

    The idea is to meter this at the L3 routers levels. The point is to allow
    operators to configure IP ranges and to assign a label to them. For example
    we will be able to set two labels; one for the internal traffic, and the
    other one for the external traffic. Each label will measure the traffic for
    a specific set of IP range. Then, bandwidth measurement will be sent for
    each label to the Oslo notification system and could be collected by
    Ceilometer.
    """

    support_status = support.SupportStatus(version='2014.1')

    entity = 'metering_label'

    PROPERTIES = (
        NAME, DESCRIPTION, SHARED,
    ) = (
        'name', 'description', 'shared',
    )

    ATTRIBUTES = (
        NAME_ATTR, DESCRIPTION_ATTR, SHARED_ATTR,
    ) = (
        'name', 'description', 'shared',
    )

    properties_schema = {
        NAME: properties.Schema(
            properties.Schema.STRING,
            _('Name of the metering label.')
        ),
        DESCRIPTION: properties.Schema(
            properties.Schema.STRING,
            _('Description of the metering label.'),
        ),
        SHARED: properties.Schema(
            properties.Schema.BOOLEAN,
            _('Whether the metering label should be shared '
              'across all tenants.'),
            default=False,
            support_status=support.SupportStatus(version='2015.1'),
        ),
    }

    attributes_schema = {
        NAME_ATTR: attributes.Schema(
            _('Name of the metering label.'),
            type=attributes.Schema.STRING
        ),
        DESCRIPTION_ATTR: attributes.Schema(
            _('Description of the metering label.'),
            type=attributes.Schema.STRING
        ),
        SHARED_ATTR: attributes.Schema(
            _('Shared status of the metering label.'),
            type=attributes.Schema.STRING
        ),
    }

    def handle_create(self):
        props = self.prepare_properties(
            self.properties,
            self.physical_resource_name())

        metering_label = self.client().create_metering_label(
            {'metering_label': props})['metering_label']

        self.resource_id_set(metering_label['id'])

    def handle_delete(self):
        try:
            self.client().delete_metering_label(self.resource_id)
        except Exception as ex:
            self.client_plugin().ignore_not_found(ex)
        else:
            return True


class MeteringRule(neutron.NeutronResource):
    """A resource to create rule for some label.

    Resource for allowing specified label to measure the traffic for a specific
    set of ip range.
    """

    support_status = support.SupportStatus(version='2014.1')

    entity = 'metering_label_rule'

    PROPERTIES = (
        METERING_LABEL_ID, REMOTE_IP_PREFIX, DIRECTION, EXCLUDED,
    ) = (
        'metering_label_id', 'remote_ip_prefix', 'direction', 'excluded',
    )

    ATTRIBUTES = (
        DIRECTION_ATTR, EXCLUDED_ATTR, METERING_LABEL_ID_ATTR,
        REMOTE_IP_PREFIX_ATTR,
    ) = (
        'direction', 'excluded', 'metering_label_id',
        'remote_ip_prefix',
    )

    properties_schema = {
        METERING_LABEL_ID: properties.Schema(
            properties.Schema.STRING,
            _('The metering label ID to associate with this metering rule.'),
            required=True
        ),
        REMOTE_IP_PREFIX: properties.Schema(
            properties.Schema.STRING,
            _('Indicates remote IP prefix to be associated with this '
              'metering rule.'),
            required=True,
        ),
        DIRECTION: properties.Schema(
            properties.Schema.STRING,
            _('The direction in which metering rule is applied, '
              'either ingress or egress.'),
            default='ingress',
            constraints=[constraints.AllowedValues((
                'ingress', 'egress'))]
        ),
        EXCLUDED: properties.Schema(
            properties.Schema.BOOLEAN,
            _('Specify whether the remote_ip_prefix will be excluded or '
              'not from traffic counters of the metering label. For example '
              'to not count the traffic of a specific IP address of a range.'),
            default='False'
        )
    }

    attributes_schema = {
        DIRECTION_ATTR: attributes.Schema(
            _('The direction in which metering rule is applied.'),
            type=attributes.Schema.STRING
        ),
        EXCLUDED_ATTR: attributes.Schema(
            _('Exclude state for cidr.'),
            type=attributes.Schema.STRING
        ),
        METERING_LABEL_ID_ATTR: attributes.Schema(
            _('The metering label ID to associate with this metering rule.'),
            type=attributes.Schema.STRING
        ),
        REMOTE_IP_PREFIX_ATTR: attributes.Schema(
            _('CIDR to be associated with this metering rule.'),
            type=attributes.Schema.STRING
        ),
    }

    def handle_create(self):
        props = self.prepare_properties(
            self.properties,
            self.physical_resource_name())

        metering_label_rule = self.client().create_metering_label_rule(
            {'metering_label_rule': props})['metering_label_rule']

        self.resource_id_set(metering_label_rule['id'])

    def handle_delete(self):
        try:
            self.client().delete_metering_label_rule(self.resource_id)
        except Exception as ex:
            self.client_plugin().ignore_not_found(ex)
        else:
            return True


def resource_mapping():
    return {
        'OS::Neutron::MeteringLabel': MeteringLabel,
        'OS::Neutron::MeteringRule': MeteringRule,
    }



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_log import versionutils
from oslo_policy import policy

from heat.policies import base

POLICY_ROOT = 'actions:%s'

DEPRECATED_REASON = """
The actions API now supports system scope and default roles.
"""

deprecated_action = policy.DeprecatedRule(
    name=POLICY_ROOT % 'action',
    check_str=base.RULE_DENY_STACK_USER,
    deprecated_reason=DEPRECATED_REASON,
    deprecated_since=versionutils.deprecated.WALLABY
)
deprecated_snapshot = policy.DeprecatedRule(
    name=POLICY_ROOT % 'snapshot',
    check_str=base.RULE_DENY_STACK_USER,
    deprecated_reason=DEPRECATED_REASON,
    deprecated_since=versionutils.deprecated.WALLABY
)
deprecated_suspend = policy.DeprecatedRule(
    name=POLICY_ROOT % 'suspend',
    check_str=base.RULE_DENY_STACK_USER,
    deprecated_reason=DEPRECATED_REASON,
    deprecated_since=versionutils.deprecated.WALLABY
)
deprecated_resume = policy.DeprecatedRule(
    name=POLICY_ROOT % 'resume',
    check_str=base.RULE_DENY_STACK_USER,
    deprecated_reason=DEPRECATED_REASON,
    deprecated_since=versionutils.deprecated.WALLABY
)
deprecated_check = policy.DeprecatedRule(
    name=POLICY_ROOT % 'check',
    check_str=base.RULE_DENY_STACK_USER,
    deprecated_reason=DEPRECATED_REASON,
    deprecated_since=versionutils.deprecated.WALLABY
)
deprecated_cancel_update = policy.DeprecatedRule(
    name=POLICY_ROOT % 'cancel_update',
    check_str=base.RULE_DENY_STACK_USER,
    deprecated_reason=DEPRECATED_REASON,
    deprecated_since=versionutils.deprecated.WALLABY
)
deprecated_cancel_without_rollback = policy.DeprecatedRule(
    name=POLICY_ROOT % 'cancel_without_rollback',
    check_str=base.RULE_DENY_STACK_USER,
    deprecated_reason=DEPRECATED_REASON,
    deprecated_since=versionutils.deprecated.WALLABY
)


actions_policies = [
    policy.DocumentedRuleDefault(
        name=POLICY_ROOT % 'action',
        check_str=base.SYSTEM_ADMIN_OR_PROJECT_MEMBER,
        description='Performs non-lifecycle operations on the stack '
        '(Snapshot, Resume, Cancel update, or check stack resources). '
        'This is the default for all actions but can be overridden by more '
        'specific policies for individual actions.',
        operations=[{
            'path': '/v1/{tenant_id}/stacks/{stack_name}/{stack_id}/actions',
            'method': 'POST',
        }],
        deprecated_rule=deprecated_action
    ),
    policy.DocumentedRuleDefault(
        name=POLICY_ROOT % 'snapshot',
        check_str=base.SYSTEM_ADMIN_OR_PROJECT_MEMBER,
        scope_types=['system', 'project'],
        description='Create stack snapshot',
        operations=[{
            'path': '/v1/{tenant_id}/stacks/{stack_name}/{stack_id}/actions',
            'method': 'POST',
        }],
        deprecated_rule=deprecated_snapshot
    ),
    policy.DocumentedRuleDefault(
        name=POLICY_ROOT % 'suspend',
        check_str=base.SYSTEM_ADMIN_OR_PROJECT_MEMBER,
        scope_types=['system', 'project'],
        description='Suspend a stack.',
        operations=[{
            'path': '/v1/{tenant_id}/stacks/{stack_name}/{stack_id}/actions',
            'method': 'POST',
        }],
        deprecated_rule=deprecated_suspend
    ),
    policy.DocumentedRuleDefault(
        name=POLICY_ROOT % 'resume',
        check_str=base.SYSTEM_ADMIN_OR_PROJECT_MEMBER,
        scope_types=['system', 'project'],
        description='Resume a suspended stack.',
        operations=[{
            'path': '/v1/{tenant_id}/stacks/{stack_name}/{stack_id}/actions',
            'method': 'POST',
        }],
        deprecated_rule=deprecated_resume
    ),
    policy.DocumentedRuleDefault(
        name=POLICY_ROOT % 'check',
        check_str=base.SYSTEM_OR_PROJECT_READER,
        scope_types=['system', 'project'],
        description='Check stack resources.',
        operations=[{
            'path': '/v1/{tenant_id}/stacks/{stack_name}/{stack_id}/actions',
            'method': 'POST',
        }],
        deprecated_rule=deprecated_check
    ),
    policy.DocumentedRuleDefault(
        name=POLICY_ROOT % 'cancel_update',
        check_str=base.SYSTEM_ADMIN_OR_PROJECT_MEMBER,
        scope_types=['system', 'project'],
        description='Cancel stack operation and roll back.',
        operations=[{
            'path': '/v1/{tenant_id}/stacks/{stack_name}/{stack_id}/actions',
            'method': 'POST',
        }],
        deprecated_rule=deprecated_cancel_update
    ),
    policy.DocumentedRuleDefault(
        name=POLICY_ROOT % 'cancel_without_rollback',
        check_str=base.SYSTEM_ADMIN_OR_PROJECT_MEMBER,
        scope_types=['system', 'project'],
        description='Cancel stack operation without rolling back.',
        operations=[{
            'path': '/v1/{tenant_id}/stacks/{stack_name}/{stack_id}/actions',
            'method': 'POST',
        }],
        deprecated_rule=deprecated_cancel_without_rollback
    )
]


def list_rules():
    return actions_policies
